/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2018 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "strings.h"


tUChar* StringAddString(tUChar* bufferP, const tChar* stringP)
{
  // Until end of null terminated string
  while (*stringP != '\0')
  {
    *(bufferP++) = *(stringP++);
  }

  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


tUChar* StringAddChar(tUChar* bufferP, tChar character)
{
  *(bufferP++) = character;

  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


tUChar* StringAddSpace(tUChar* bufferP)
{
  *(bufferP++) = ' ';

  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


tUChar* StringAddDecByte(tUChar* bufferP, tUChar Value)
{
tUChar Temp;
tUChar Divisor;
tBool FirstZero;

  Divisor = 100U;
  FirstZero = cTrue;
  
  while (Divisor >= 10U)
  {
    Temp = Value / Divisor;
    Value -= (tUChar)(Temp * Divisor);
    // Prevent leading zero.
    if ( (Temp > 0U) || (FirstZero == cFalse) )
    {
      *(bufferP++) = (tUChar)'0' + Temp;
      FirstZero = cFalse;
    }
    
    Divisor /= 10U;
  }
  
  // Add Unity
  *(bufferP++) = (tUChar)'0' + Value;

  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


tUChar* StringAddHexByte(tUChar* bufferP, tUChar Byte)
{
tUChar Nibble;

  // Most significant Nibble
  Nibble = (Byte >> 4) & 0x0F;
  *(bufferP++) = (Nibble >= 0x0A) ? 'A' + (Nibble - 0x0A) : '0' + Nibble;
  
  // Least Significant Nibble
  Nibble = Byte & 0x0F;
  *(bufferP++) = (Nibble >= 0x0A) ? 'A' + (Nibble - 0x0A) : '0' + Nibble;
  
  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


tUChar* StringAddHexShort(tUChar* bufferP, tUShort Short)
{
tUShort Nibble;

  // Most significant Nibble
  Nibble = (Short >> 12) & 0x000F;
  *(bufferP++) = (Nibble >= 0x000A) ? 'A' + (Nibble - 0x000A) : '0' + Nibble;

  Nibble = (Short >> 8) & 0x000F;
  *(bufferP++) = (Nibble >= 0x000A) ? 'A' + (Nibble - 0x000A) : '0' + Nibble;
  
  Nibble = (Short >> 4) & 0x000F;
  *(bufferP++) = (Nibble >= 0x000A) ? 'A' + (Nibble - 0x000A) : '0' + Nibble;
  
  // Least Significant Nibble
  Nibble = Short & 0x000F;
  *(bufferP++) = (Nibble >= 0x000A) ? 'A' + (Nibble - 0x000A) : '0' + Nibble;

  // Terminate string
  *bufferP = '\0';

  return bufferP;
}
//---------------------------------------------------------------------------


#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif

