/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2015-2019 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "..\Native_CPC.h"


/*
** Make sure we can call this stuff from C++.
*/
#ifdef __cplusplus
extern "C" {
#endif


//===================
// PATCH begin
#ifdef _PATCH_ENABLE

#endif /* _PATCH_ENABLE */
// PATCH end
//===================


extern tULong AMSDOS_GetFirstSectorOfBlock(tDiskOperation* DiskOperation, tULong Block);
extern tULong AMSDOS_GetNextSector(tDiskOperation* DiskOperation, tULong CHRN);
extern tSector* AMSDOS_FindSector(tDiskOperation* DiskOperation, tULong CHRN);
extern tULong ROMDOS_GetFirstSectorOfBlock(tDiskOperation* DiskOperation, tULong Block);
extern tULong ROMDOS_GetNextSector(tDiskOperation* DiskOperation, tULong CHRN);
extern tSector* ROMDOS_FindSector(tDiskOperation* DiskOperation, tULong CHRN);


tSector* Native_DiskGetNextSectorOfBlock(tDiskOperation* DiskOperationP, tULong Block)
/***********************************************************************
 *
 *  Native_DiskGetNextSectorOfBlock
 *
 ***********************************************************************/
{
tSector* SectorP = cNull;

  // Detect format type
  if (DiskOperationP->FormatType == DISK_FORMAT_UNKNOWN)
  {
    Native_DiskFormatType(DiskOperationP);
  }

  switch (DiskOperationP->FormatType)
  {
    // ROMDOS
    case DISK_FORMAT_ROMDOS_D10:
    {
      tULong FirstCHRN = ROMDOS_GetFirstSectorOfBlock(DiskOperationP, Block);
      tULong SecondCHRN = ROMDOS_GetNextSector(DiskOperationP, FirstCHRN);
      tULong ThirdCHRN = ROMDOS_GetNextSector(DiskOperationP, SecondCHRN);
      tULong FourthCHRN = ROMDOS_GetNextSector(DiskOperationP, ThirdCHRN);
      if (DiskOperationP->CatalogueLastCHRN == FirstCHRN)
      {
        DiskOperationP->CatalogueLastCHRN = SecondCHRN;
	      SectorP = AMSDOS_FindSector(DiskOperationP, DiskOperationP->CatalogueLastCHRN);
      }
      else if (DiskOperationP->CatalogueLastCHRN == SecondCHRN)
      {
        DiskOperationP->CatalogueLastCHRN = ThirdCHRN;
	      SectorP = AMSDOS_FindSector(DiskOperationP, DiskOperationP->CatalogueLastCHRN);
      }
      else if (DiskOperationP->CatalogueLastCHRN == ThirdCHRN)
      {
        DiskOperationP->CatalogueLastCHRN = FourthCHRN;
	      SectorP = AMSDOS_FindSector(DiskOperationP, DiskOperationP->CatalogueLastCHRN);
      }
      else
      {
        DiskOperationP->CatalogueLastCHRN = 0;
      }
    }
    break;

	  // AMSDOS
    case DISK_FORMAT_AMSDOS_DATA:
    case DISK_FORMAT_AMSDOS_SYSTEM:
    case DISK_FORMAT_AMSDOS_IBM:
	  {
      tULong FirstCHRN = AMSDOS_GetFirstSectorOfBlock(DiskOperationP, Block);
      tULong SecondCHRN = AMSDOS_GetNextSector(DiskOperationP, FirstCHRN);
      if (DiskOperationP->CatalogueLastCHRN == FirstCHRN)
      {
        DiskOperationP->CatalogueLastCHRN = SecondCHRN;
	      SectorP = AMSDOS_FindSector(DiskOperationP, DiskOperationP->CatalogueLastCHRN);
      }
      else
      {
        DiskOperationP->CatalogueLastCHRN = 0;
      }
	  }
    break;
  }

  return (SectorP);
}
/*----------------------------------------------------------------------------*/

#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif


