/*
 CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
 Copyright (C) 2014-2018 by Frdric Coste

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <vcl.h>
#include <stdlib.h>
#include <Clipbrd.hpp>
#include <tchar.h>
#pragma hdrstop
#include "TFormCodeExplorer.h"
#include "TFormEditValue.h"
#include "TFormMain.h"
#include "..\..\Common\Z80DASM\z80dasm.h"
#include "..\..\Common\Pasmo\pasmo.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"


//=================================
// Options

//=================================


static const AnsiString cTitle = "Code explorer";
static const AnsiString cTitlePaused  = " - [PAUSED]";


#define NB_DIGITS_TOP_ADDRESS_HEX         4
#define NB_DIGITS_TOP_ADDRESS_DEC         5
#define NB_DIGITS_CODE_ORIGIN_HEX         4
#define NB_DIGITS_CODE_ORIGIN_DEC         5
#define NB_DIGITS_CODE_SIZE_HEX           4
#define NB_DIGITS_CODE_SIZE_DEC           5



static tUChar ReadFileData(tNativeCPC* NativeCPC,tULong addr)
{
  return (((tUChar*)NativeCPC)[addr]);
}


TFormCodeExplorer *FormCodeExplorer;
//---------------------------------------------------------------------------
__fastcall TFormCodeExplorer::TFormCodeExplorer(TComponent* Owner)
  : TForm(Owner)
{
  mEmulatorP = NULL;
  mSettingsP = NULL;

  mTopAddress = 0;
  mCodeOrigin = 0;
  mCodeLast = 0;
  mCodeSize = 1;
  mSelectedAddress = 0;

  mAssemblerBASICFilename = "CODE.BIN";
  ComboBoxCodeFilename->Text = mAssemblerBASICFilename;

  PageControlFeatures->ActivePage = TabSheetAssembler;

  PanelButtonPatchUpdate->Height = EditPatchAssembly->Height;

  // Apply Theme
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("Panel") == 1)
    {
      ((TPanel*)Components[Loop])->Font->Color = clBlack;
    }
    else if (Components[Loop]->Name.Pos("Edit") == 1)
    {
      ((TEdit*)Components[Loop])->Font->Color = clBlack;
    }
    else if (Components[Loop]->Name.Pos("Memo") == 1)
    {
      ((TMemo*)Components[Loop])->Font->Color = clBlack;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::FormDestroy(TObject *Sender)
{
  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::FormShow(TObject *Sender)
{
  if (FormMain == NULL)
  {
    Close();
    return;
  }

  ShowHint = FormMain->ShowHint;

  TabControlMemoryContent->TabIndex = 0;
  mMemoryStartAddress = 0x0000;
  mMemorySize = 0x10000;
  mMaxAddress = mMemoryStartAddress + mMemorySize - 1;
  mZ80dasmParam.NativeCPC = FormMain->NativeCPC;
  mZ80dasmParam.ReadFuntion = Engine_ReadRAM;
  mWriteFunction = Engine_WriteRAM;

  StringGridInstructions->RowCount = StringGridInstructions->Height / StringGridInstructions->DefaultRowHeight;
  StringGridInstructions->ColWidths[0] = 48;
  StringGridInstructions->ColWidths[1] = 100;
  StringGridInstructions->ColWidths[2] = StringGridInstructions->Width -
        (StringGridInstructions->ColWidths[0]+StringGridInstructions->ColWidths[1]);
  ScrollBarAddress->PageSize = StringGridInstructions->RowCount;

  PanelFormatTopAddress->Caption = "&&";
  TopAddress = mTopAddress;
  PanelFormatCodeOrigin->Caption = "&&";
  CodeOrigin = mCodeOrigin;
  PanelFormatCodeLast->Caption = "&&";
  CodeLast = mCodeLast;
  PanelFormatCodeSize->Caption = "&&";
  CodeSize = mCodeSize;

  UpdateCatalogue();

  if (mSettingsP)
  {
    mSettingsP->CodeExplorerFormVisible = true;
  }

  // Subscribe to events
  FormMain->SubscribeToDiskEvents(OnDiskEvent);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::FormHide(TObject *Sender)
{
  // Unsubscribe to events
  if (FormMain != NULL)
  {
    FormMain->UnsubscribeToDiskEvents(OnDiskEvent);
  }

  if (mSettingsP)
  {
    mSettingsP->CodeExplorerFormVisible = false;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


AnsiString __fastcall TFormCodeExplorer::GetTitle(void)
{
  AnsiString Title = cTitle;
  if (mPaused == true) Title += cTitlePaused;

  return Title;
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::SetSettings(TSettings* Value)
{
  mSettingsP = Value;

  if (    (mSettingsP->CodeExplorerFormLeft != 0)
       || (mSettingsP->CodeExplorerFormTop != 0)
       || (mSettingsP->CodeExplorerFormWidth != 0)
       || (mSettingsP->CodeExplorerFormHeight != 0) )
  {
    Position = poDesigned;
    Left = mSettingsP->CodeExplorerFormLeft;
    Top = mSettingsP->CodeExplorerFormTop;
    Width = mSettingsP->CodeExplorerFormWidth;
    Height = mSettingsP->CodeExplorerFormHeight;
  }

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  Font->Color = mSettingsP->ThemeText1Color;
  StringGridInstructions->Font->Color = mSettingsP->ThemeText2Color;
  PanelPatchMachine->Font->Color = mSettingsP->ThemeText1Color;
  PanelCodeSize->Font->Color = mSettingsP->ThemeText1Color;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("PanelButton") == 1)
    {
      ((TPanel*)Components[Loop])->Color = mSettingsP->ThemeButtonColor;
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeButtonTextColor;
    }
    else if (Components[Loop]->Name.Pos("Label") == 1)
    {
      ((TLabel*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
    else if (Components[Loop]->Name.Pos("EditNoteAddress") == 1)
    {
      ((TEdit*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
  }

  // Refresh display
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (((TControl*)Components[Loop])->Visible == true)
    {
      ((TControl*)Components[Loop])->Invalidate();
    }
  }

  // Get work data
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    AnsiString PanelMarkerName("PanelAddress");
    AnsiString EditNoteMarkerName("EditNoteAddress");
    if (Components[Loop]->Name.Pos(PanelMarkerName) == 1)
    {
      TPanel* MarkerPanel = (TPanel*)Components[Loop];
      int Index = MarkerPanel->Name.SubString(PanelMarkerName.Length()+1,
                  MarkerPanel->Name.Length() - PanelMarkerName.Length()).ToInt();
      // Set last marker
      MarkerPanel->Caption = mSettingsP->GetCodeExplorerMarker(Index-1);
    }
    else if (Components[Loop]->Name.Pos(EditNoteMarkerName) == 1)
    {
      TEdit* MarkerLabel = (TEdit*)Components[Loop];
      int Index = MarkerLabel->Name.SubString(EditNoteMarkerName.Length()+1,
                  MarkerLabel->Name.Length() - EditNoteMarkerName.Length()).ToInt();
      // Set last marker
      MarkerLabel->Text = mSettingsP->GetCodeExplorerMarkerLabel(Index-1);
    }
  }

  if (mSettingsP->HideAll == false)
  {
    Visible = mSettingsP->CodeExplorerFormVisible;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::FormResize(TObject *Sender)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->CodeExplorerFormWidth = Width;
    mSettingsP->CodeExplorerFormHeight = Height;
  }

  if (Visible == true)
  {
    StringGridInstructions->SetFocus();
  }

  StringGridInstructions->RowCount = StringGridInstructions->Height / StringGridInstructions->DefaultRowHeight;
  StringGridInstructions->ColWidths[2] = StringGridInstructions->Width -
        (StringGridInstructions->ColWidths[0]+StringGridInstructions->ColWidths[1]);
  ScrollBarAddress->PageSize = StringGridInstructions->RowCount;

  if ( (FormMain->NativeCPC != NULL) && (Visible == true) )
  {
    UpdateInstructions(FormMain->NativeCPC);
    UpdatePatch(FormMain->NativeCPC);
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::WMMove(TMessage &Message)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->CodeExplorerFormLeft = Left;
    mSettingsP->CodeExplorerFormTop = Top;
  }

  NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::UpdateInstructions(tNativeCPC* NativeCPC)
{
tULong addr = mTopAddress;

  // Update UpperROM Index
  TabControlMemoryContent->Tabs->Strings[2] = "Upper ROM " + AnsiString(NativeCPC->GateArray->upper_ROM);

  for (int Loop=0; Loop < StringGridInstructions->RowCount; Loop++)
  {
    if (addr <= mMaxAddress)
    {
      StringGridInstructions->Cells[0][Loop] = AnsiString::IntToHex(addr,NB_DIGITS_TOP_ADDRESS_HEX) + ":";
      addr = Z80dasmDecodeInstruction(addr,
                                      &mZ80dasmParam);

      if (addr <= (mMaxAddress+1))
      {
        StringGridInstructions->Cells[1][Loop] = AnsiString(mZ80dasmParam.MachineCode).UpperCase();
        StringGridInstructions->Cells[2][Loop] = AnsiString(mZ80dasmParam.Instruction).UpperCase();
      }
      else
      {
        StringGridInstructions->Cells[1][Loop] = AnsiString(mZ80dasmParam.MachineCode).SubString(0,3).UpperCase();
        StringGridInstructions->Cells[2][Loop] = "ILLEGAL";
      }
    }
    else
    {
      StringGridInstructions->Cells[0][Loop] = "";
      StringGridInstructions->Cells[1][Loop] = "";
      StringGridInstructions->Cells[2][Loop] = "";
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::UpdateCatalogue(void)
{
  // Populate files list
  AnsiString LastFilename = ComboBoxCodeFilename->Text;
  ComboBoxCodeFilename->Clear();
  mEmulatorP->ReadDiskCatalogue(DriveA, (tUChar)0);
  for (int Loop=0; Loop < mEmulatorP->NbCatalogEntries; Loop++)
  {
    ComboBoxCodeFilename->Items->Add(mEmulatorP->CatalogEntry[Loop]);
  }

  ComboBoxCodeFilename->Text = LastFilename;
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::UpdatePatch(tNativeCPC* NativeCPC)
{
tULong addr = mTopAddress;
tULong instr_addr = addr;

  addr = Z80dasmDecodeInstruction(addr,
                                  &mZ80dasmParam);

  int Loop;
  for (Loop=0;
       ((Loop < StringGridInstructions->Row) && (addr <= mMaxAddress));
       Loop++)
  {
    instr_addr = addr;
    addr = Z80dasmDecodeInstruction(addr,
                                    &mZ80dasmParam);
  }

  if ( (Loop == StringGridInstructions->Row)
       && (instr_addr >= mMemoryStartAddress) && (instr_addr <= mMaxAddress) )
  {
    mSelectedAddress = instr_addr;
    EditPatchAssembly->Text = AnsiString(mZ80dasmParam.Instruction).UpperCase();
  }
  else
  {
    EditPatchAssembly->Text = "";
  }

  NOT_USED(NativeCPC);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::StringGridInstructionsDrawCell(TObject *Sender,
          int ACol, int ARow, TRect &Rect, TGridDrawState State)
{
  TStringGrid* StringGrid = (TStringGrid*)Sender;

  if (State.Contains(gdSelected))
  {
    StringGrid->Canvas->Brush->Color = clHighlight;
    StringGrid->Canvas->Font->Color = clWhite;
  }
  else if (ACol == 0)
  {
    StringGrid->Canvas->Brush->Color = Color;
    StringGrid->Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }
  else
  {
    StringGrid->Canvas->Brush->Color = Color;
    StringGrid->Canvas->Font->Color = mSettingsP->ThemeText1Color;
  }

  // Draw cell
  StringGrid->Canvas->FillRect(Rect);
  int TextHeight = StringGrid->Canvas->TextHeight(StringGrid->Cells[ACol][ARow]);
  int TextPos = (StringGrid->RowHeights[ARow] - TextHeight) / 2;
  StringGrid->Canvas->TextOut(Rect.Left+5,
                              Rect.Top+TextPos,
                              StringGrid->Cells[ACol][ARow]);

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelFormatTopAddressClick(TObject *Sender)
{
  if (PanelFormatTopAddress->Caption.IsEmpty() == true)
  {
    PanelFormatTopAddress->Caption = "&&";
    PanelTopAddress->Caption = AnsiString::IntToHex(mTopAddress, NB_DIGITS_TOP_ADDRESS_HEX);
  }
  else
  {
    PanelFormatTopAddress->Caption = "";
    PanelTopAddress->Caption = mTopAddress;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelTopAddressClick(TObject *Sender)
{
  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mTopAddress;
  EditValue->Hexadecimal = PanelFormatTopAddress->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = PanelFormatTopAddress->Caption.IsEmpty() == true ? NB_DIGITS_TOP_ADDRESS_DEC : NB_DIGITS_TOP_ADDRESS_HEX;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mMaxAddress;
  if (EditValue->ShowModal() == mrOk)
  {
    TopAddress = EditValue->NewValue;
    StringGridInstructions->SetFocus();
    StringGridInstructions->Row = 0;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::SetTopAddress(tULong Value)
{
  mTopAddress = Value;
  if (mTopAddress < mMemoryStartAddress)
  {
    mTopAddress = mMemoryStartAddress;
  }
  else if (mTopAddress > mMaxAddress)
  {
    mTopAddress = mMaxAddress;
  }
  ScrollBarAddress->Position = mTopAddress;

  if (PanelFormatTopAddress->Caption.IsEmpty() == true)
  {
    PanelTopAddress->Caption = mTopAddress;
  }
  else
  {
    PanelTopAddress->Caption = AnsiString::IntToHex(mTopAddress, NB_DIGITS_TOP_ADDRESS_HEX);
  }

  UpdateInstructions(FormMain->NativeCPC);
  UpdatePatch(FormMain->NativeCPC);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::SetCodeOrigin(tLong Value)
{
  mCodeOrigin = Value;

  // Get size of last instruction
  tLong addr = Z80dasmDecodeInstruction(CodeLast, &mZ80dasmParam);
  CodeSize = addr - CodeOrigin;

  if (PanelFormatCodeOrigin->Caption.IsEmpty() == true)
  {
    PanelCodeOrigin->Caption = mCodeOrigin;
  }
  else
  {
    PanelCodeOrigin->Caption = AnsiString::IntToHex(mCodeOrigin, NB_DIGITS_CODE_ORIGIN_HEX);
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelFormatCodeOriginClick(TObject *Sender)
{
  if (PanelFormatCodeOrigin->Caption.IsEmpty() == true)
  {
    PanelFormatCodeOrigin->Caption = "&&";
    PanelCodeOrigin->Caption = AnsiString::IntToHex(mCodeOrigin, NB_DIGITS_CODE_ORIGIN_HEX);
  }
  else
  {
    PanelFormatCodeOrigin->Caption = "";
    PanelCodeOrigin->Caption = mCodeOrigin;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelCodeOriginClick(TObject *Sender)
{
  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mCodeOrigin;
  EditValue->Hexadecimal = PanelFormatCodeOrigin->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = PanelFormatCodeOrigin->Caption.IsEmpty() == true ? NB_DIGITS_CODE_ORIGIN_DEC : NB_DIGITS_CODE_ORIGIN_HEX;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mMaxAddress;
  if (EditValue->ShowModal() == mrOk)
  {
    CodeOrigin = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::SetCodeLast(tLong Value)
{
  mCodeLast = Value;

  // Get size of last instruction
  tLong addr = Z80dasmDecodeInstruction(CodeLast, &mZ80dasmParam);
  CodeSize = addr - CodeOrigin;

  if (PanelFormatCodeLast->Caption.IsEmpty() == true)
  {
    PanelCodeLast->Caption = mCodeLast;
  }
  else
  {
    PanelCodeLast->Caption = AnsiString::IntToHex(mCodeLast, NB_DIGITS_CODE_ORIGIN_HEX);
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelFormatCodeLastClick(TObject *Sender)
{
  if (PanelFormatCodeLast->Caption.IsEmpty() == true)
  {
    PanelFormatCodeLast->Caption = "&&";
    PanelCodeLast->Caption = AnsiString::IntToHex(mCodeLast, NB_DIGITS_CODE_ORIGIN_HEX);
  }
  else
  {
    PanelFormatCodeLast->Caption = "";
    PanelCodeLast->Caption = mCodeLast;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelCodeLastClick(TObject *Sender)
{
  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mCodeLast;
  EditValue->Hexadecimal = PanelFormatCodeLast->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = PanelFormatCodeLast->Caption.IsEmpty() == true ? NB_DIGITS_CODE_ORIGIN_DEC : NB_DIGITS_CODE_ORIGIN_HEX;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mMaxAddress;
  if (EditValue->ShowModal() == mrOk)
  {
    CodeLast = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------



void __fastcall TFormCodeExplorer::SetCodeSize(tLong Value)
{
  mCodeSize = Value;

  if (mCodeSize < 0)
  {
    PanelCodeSize->Caption = "-";
  }
  else
  {
    if (PanelFormatCodeSize->Caption.IsEmpty() == true)
    {
      PanelCodeSize->Caption = mCodeSize;
    }
    else
    {
      PanelCodeSize->Caption = AnsiString::IntToHex(mCodeSize, NB_DIGITS_CODE_SIZE_HEX);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelFormatCodeSizeClick(TObject *Sender)
{
  if (PanelFormatCodeSize->Caption.IsEmpty() == true)
  {
    PanelFormatCodeSize->Caption = "&&";
    PanelCodeSize->Caption = AnsiString::IntToHex(mCodeSize, NB_DIGITS_CODE_SIZE_HEX);
  }
  else
  {
    PanelFormatCodeSize->Caption = "";
    PanelCodeSize->Caption = mCodeSize;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::ScrollBarAddressChange(TObject *Sender)
{
  TopAddress = ScrollBarAddress->Position;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonRefreshClick(TObject *Sender)
{
  UpdateInstructions(FormMain->NativeCPC);
  UpdatePatch(FormMain->NativeCPC);
  UpdateCatalogue();
  StringGridInstructions->SetFocus();
  // Refresh code size
  CodeLast = CodeLast;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonRegPCClick(TObject *Sender)
{
tNativeCPC* NativeCPC = FormMain->NativeCPC;
tZ80* Z80 = NativeCPC->Z80;

  // Lower ROM ?
  if ( (_PC < 0x4000) && (NativeCPC->membank_read[0] == NativeCPC->pbROMlo) )
  {
    // Select Lower ROM
    MemoryIndex = 1;
  }
  // Upper ROM ?
  else if ( (_PC >= 0xC000) && (NativeCPC->membank_read[3] == NativeCPC->pbExpansionROM) )
  {
    // Select Upper ROM
    MemoryIndex = 2;
  }
  else
  {
    // Select RAM
    MemoryIndex = 0;
  }


  TopAddress = _PC;
  StringGridInstructions->SetFocus();
  StringGridInstructions->Row = 0;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::StringGridInstructionsMouseWheelUp(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if (!StringGridInstructions->Row)
  {
    if (TopAddress > mMemoryStartAddress)
    {
      TopAddress--;
    }
  }

	NOT_USED(Sender);
	NOT_USED(Shift);
	NOT_USED(MousePos);
	NOT_USED(Handled);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::StringGridInstructionsMouseWheelDown(TObject *Sender,
          TShiftState Shift, TPoint &MousePos, bool &Handled)
{
  if (StringGridInstructions->Row == StringGridInstructions->RowCount-1)
  {
    if (TopAddress < mMaxAddress)
    {
      TopAddress++;
    }
  }

	NOT_USED(Sender);
	NOT_USED(Shift);
	NOT_USED(MousePos);
	NOT_USED(Handled);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::StringGridInstructionsDblClick(TObject *Sender)

{
tULong addr = mTopAddress;

  for (int Loop=0;
       ((Loop < StringGridInstructions->Row) && (addr <= mMaxAddress));
       Loop++)
  {
    addr = Z80dasmDecodeInstruction(addr,
                                    &mZ80dasmParam);
  }

  if (addr <= mMaxAddress)
  {
    TopAddress = addr;
    StringGridInstructions->SetFocus();
    StringGridInstructions->Row = 0;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonSetCodeOriginClick(TObject *Sender)
{
tULong addr = mTopAddress;

  for (int Loop=0;
       ((Loop < StringGridInstructions->Row) && (addr < mMaxAddress));
       Loop++)
  {
    addr = Z80dasmDecodeInstruction(addr,
                                    &mZ80dasmParam);
  }

  if (addr <= mMaxAddress)
  {
    CodeOrigin = addr;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelButtonGoToCodeOriginClick(TObject *Sender)
{
  TopAddress = CodeOrigin;
  StringGridInstructions->SetFocus();
  StringGridInstructions->Row = 0;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonSetCodeLastClick(TObject *Sender)
{
tULong addr = mTopAddress;

  for (int Loop=0;
       ((Loop < StringGridInstructions->Row) && (addr < mMaxAddress));
       Loop++)
  {
    addr = Z80dasmDecodeInstruction(addr, &mZ80dasmParam);
  }

  if (addr <= mMaxAddress)
  {
    CodeLast = addr;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelButtonGoToCodeLastClick(TObject *Sender)
{
  TopAddress = CodeLast;
  StringGridInstructions->SetFocus();
  StringGridInstructions->Row = 0;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::StringGridInstructionsClick(TObject *Sender)
{
  UpdatePatch(FormMain->NativeCPC);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::EditPatchAssemblyChange(TObject *Sender)
{
  // Get whole assembler code as long char string.
  int linesize = EditPatchAssembly->Text.Length();
  char* line = new char [linesize+1];
  size_t size = wcstombs(line, EditPatchAssembly->Text.c_str(), linesize);
  line[size] = '\0';

  unsigned char * code = NULL;

  try
  {
    if (size > 0)
    {
      unsigned short origin = mSelectedAddress;
      int codesize = pasmo_assemble_lines(line, &origin, &code);
      if (codesize > 0)
      {
        AnsiString Message = "Machine Code: ";
        for (int loop=0; loop < codesize; loop++)
        {
          Message += AnsiString::IntToHex(code[loop], 2) + " ";
        }
        PanelPatchMachine->Caption = Message;
        PanelButtonPatchUpdate->Visible = true;
      }
      else
      {
        PanelPatchMachine->Caption = "";
        PanelButtonPatchUpdate->Visible = false;
      }
    }
    else
    {
      PanelPatchMachine->Caption = "";
      PanelButtonPatchUpdate->Visible = false;
    }
  }
  catch(Exception &E)
  {
  }

  delete code;

  delete [] line;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonPatchUpdateClick(TObject *Sender)
{
  // Get whole assembler code as long char string.
  int linesize = EditPatchAssembly->Text.Length();
  char* line = new char [linesize+1];
  size_t size = wcstombs(line, EditPatchAssembly->Text.c_str(), linesize);
  line[size] = '\0';

  unsigned char * code = NULL;

  try
  {
    unsigned short origin = mSelectedAddress;
    int codesize = pasmo_assemble_lines(line, &origin, &code);
    if (codesize > 0)
    {
      // Update memory
      for (int loop=0; loop < codesize; loop++)
      {
        mWriteFunction(FormMain->NativeCPC,
                       mSelectedAddress + loop - mMemoryStartAddress,
                       code[loop]);
      }
    }
  }
  catch(Exception &E)
  {
  }

  // Refresh display

  UpdateInstructions(FormMain->NativeCPC);
  StringGridInstructions->SetFocus();

  delete code;

  delete [] line;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::TabControlMemoryContentChange(TObject *Sender)
{
  MemoryIndex = TabControlMemoryContent->TabIndex;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::TabControlMemoryContentDrawTab(TCustomTabControl *Control,
          int TabIndex, const TRect &Rect, bool Active)
{
  TCanvas* Canvas = TabControlMemoryContent->Canvas;

  if (Active == true)
  {
    Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }
  else
  {
    Canvas->Font->Color = mSettingsP->ThemeText1Color;
  }

  // Draw tab
  Canvas->FillRect(Rect);
  AnsiString Text = TabControlMemoryContent->Tabs->Strings[TabIndex];
  int TextWidth = Canvas->TextWidth(Text);
  int TextHeight = Canvas->TextHeight(Text);
  int TextPosX = (Rect.Width() - TextWidth) / 2;
  int TextPosY = (TabControlMemoryContent->TabHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+TextPosX, Rect.Top+TextPosY, Text);

  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::SetMemoryIndex(int Value)
{
  TabControlMemoryContent->TabIndex = Value;

  switch (TabControlMemoryContent->TabIndex)
  {
    case 1: // Lower ROM
    {
      mMemoryStartAddress = 0x0000;
      mMemorySize = 0x4000;
      mZ80dasmParam.ReadFuntion = Engine_ReadLowerROM;
      mWriteFunction = Engine_WriteLowerROM;

    }
    break;

    case 2: // Upper ROM
    {
      mMemoryStartAddress = 0xC000;
      mMemorySize = 0x4000;
      mZ80dasmParam.ReadFuntion = Engine_ReadUpperROM;
      mWriteFunction = Engine_WriteUpperROM;
    }
    break;

    default: // Entire RAM
    {
      mMemoryStartAddress = 0x0000;
      mMemorySize = 0x10000;
      mZ80dasmParam.ReadFuntion = Engine_ReadRAM;
      mWriteFunction = Engine_WriteRAM;
    }
    break;
  }

  mZ80dasmParam.StartAddress = mMemoryStartAddress;
  mMaxAddress = mMemoryStartAddress + mMemorySize - 1;

  // Adapt Top Address
  if (TopAddress < mMemoryStartAddress)
  {
    TopAddress = mMemoryStartAddress;
  }
  else if (TopAddress >= (mMemoryStartAddress + mMemorySize))
  {
    TopAddress = mMemoryStartAddress + mMemorySize - 1;
  }
  else
  {
    UpdateInstructions(FormMain->NativeCPC);
  }
  ActiveControl = StringGridInstructions;
  StringGridInstructions->Row = 0;

  // Adapt scrollbar
  tULong OldTopAddress = mTopAddress;
  ScrollBarAddress->Min = 0;
  ScrollBarAddress->Position = 0;
  ScrollBarAddress->Max = mMaxAddress;
  ScrollBarAddress->Position = OldTopAddress;
  ScrollBarAddress->Min = mMemoryStartAddress;
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonAssemblerUpdateClick(TObject *Sender)
{
  // Get whole assembler code as long char string.
  int linessize = RichEditAssembler->Text.Length();
  char* lines = new char [linessize+1];
  size_t size = wcstombs(lines, RichEditAssembler->Text.c_str(), linessize);
  lines[size] = '\0';

  unsigned char * code = NULL;

  try
  {
    unsigned short origin = CodeOrigin;
    int codesize = pasmo_assemble_lines(lines, &origin, &code);
    if (codesize > 0)
    {
      // Update origin
      CodeOrigin = origin;

      // code size saturation
      if ((tULong)(origin + codesize) > mMaxAddress)
      {
        codesize = mMaxAddress - origin;
      }
      CodeSize = codesize;

      // Update memory
      for (int loop=0; loop < codesize; loop++)
      {
        mWriteFunction(FormMain->NativeCPC,
                       CodeOrigin + loop - mMemoryStartAddress,
                       code[loop]);
      }

      // Get Last instruction address
      tULong addr = CodeOrigin;
      while (addr < (tULong)(origin + codesize))
      {
        tULong nextaddr = Z80dasmDecodeInstruction(addr,
                                                   &mZ80dasmParam);
        if (nextaddr == (tULong)(origin + codesize))
        {
          CodeLast = addr;
        }
        addr = nextaddr;
      }
    }
    else if (codesize < 0)// Error ?
    {
      int errorline = -codesize - 1;
      CodeSize = 0;

      // Select and display error line
      RichEditAssembler->CaretPos = TPoint(0,0);
      RichEditAssembler->SelStart = RichEditAssembler->Perform(EM_LINEINDEX, errorline, 0);
      RichEditAssembler->SelLength = RichEditAssembler->Perform(EM_LINELENGTH, RichEditAssembler->SelStart, 0);
      RichEditAssembler->SetFocus();

      MessageBeep(0);
      Application->MessageBox(_T("Error during compilation"), _T("Pasmo assembler"), MB_OK | MB_ICONERROR);
    }
  }
  catch(Exception &E)
  {
    Application->MessageBox(E.Message.c_str(), _T("Pasmo"), MB_OK);
  }

  delete code;

  // Refresh display
  UpdateInstructions(FormMain->NativeCPC);

  delete [] lines;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PageControlFeaturesDrawTab(TCustomTabControl *Control,
          int TabIndex, const TRect &Rect, bool Active)
{
  TCanvas* Canvas = PageControlFeatures->Canvas;

  if (Active == true)
  {
    Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }

  // Get sheet corresponding to TabIndex
  int TabCount=0;
  TTabSheet* Sheet = NULL;
  for (int Loop=0; Loop < PageControlFeatures->PageCount; Loop++)
  {
    if (PageControlFeatures->Pages[Loop]->TabVisible == true)
    {
      if (TabCount == TabIndex)
      {
        Sheet = PageControlFeatures->Pages[Loop];
        break;
      }

      TabCount++;
    }
  }

  // Draw tab
  Canvas->FillRect(Rect);
  AnsiString Text = Sheet->Caption;
  int TextWidth = Canvas->TextWidth(Text);
  int TextHeight = Canvas->TextHeight(Text);
  int TextPosX = (Rect.Width() - TextWidth) / 2;
  int TextPosY = (PageControlFeatures->TabHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+TextPosX, Rect.Top+TextPosY, Text);

  NOT_USED(Control);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelButtonCopyClick(TObject *Sender)
{
  tULong addr = CodeOrigin;
  tULong endaddr = CodeOrigin + CodeSize;

  TCursor mOldCursor = Screen->Cursor;
  Screen->Cursor = crHourGlass;

  RichEditAssembler->SetFocus();
  RichEditAssembler->Lines->BeginUpdate();
  while (addr < endaddr)
  {
    addr = Z80dasmDecodeInstruction(addr,
                                    &mZ80dasmParam);
    RichEditAssembler->SelText = AnsiString(mZ80dasmParam.Instruction).LowerCase() + "\r\n";
  }
  RichEditAssembler->Lines->EndUpdate();
  Screen->Cursor = mOldCursor;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelButtonAssemblerLoadClick(TObject *Sender)
{
  if (OpenDialogAssembler->Execute() == true)
  {
    RichEditAssembler->Clear();
    RichEditAssembler->Lines->LoadFromFile(OpenDialogAssembler->FileName);
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelButtonAssemblerSaveClick(TObject *Sender)
{
  if (OpenDialogAssembler->FileName.IsEmpty() == false)
  {
    SaveDialogAssembler->FileName = OpenDialogAssembler->FileName;
  }
  else
  {
    SaveDialogAssembler->FileName = "code.asm";
  }
  if (SaveDialogAssembler->Execute() == true)
  {
    RichEditAssembler->Lines->SaveToFile(SaveDialogAssembler->FileName);
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::RichEditAssemblerKeyDown(TObject *Sender, WORD &Key,
          TShiftState Shift)
{
  if (Shift.Contains(ssCtrl))
  {
    switch (Key)
    {
      case 'A':
      {
        SelectAllMenuClick(Sender);
      }
      break;

      case 'C':
      {
        CopyMenuClick(Sender);
      }
      break;

      case 'V':
      {
        PasteMenuClick(Sender);
      }
      break;
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::ClearMenuClick(TObject *Sender)
{
  bool ClearOK = true;
  if (RichEditAssembler->Text.Length() > 0)
  {
    if (Application->MessageBox(_T("Are you sure to erase your code ?"),
                                _T("Clear code"),
                                MB_YESNO | MB_ICONQUESTION) == IDNO)
    {
       ClearOK = false;
    }
  }

  if (ClearOK == true)
  {
    RichEditAssembler->Clear();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::SelectAllMenuClick(TObject *Sender)
{
  RichEditAssembler->SelectAll();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::CopyMenuClick(TObject *Sender)
{
  Clipboard()->AsText = RichEditAssembler->SelText;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PasteMenuClick(TObject *Sender)
{
  RichEditAssembler->SelText = Clipboard()->AsText;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PageControlFeaturesChange(TObject *Sender)
{
  if (PageControlFeatures->ActivePage == TabSheetPatch)
  {
    EditPatchAssembly->SetFocus();
    EditPatchAssembly->SelStart = EditPatchAssembly->Text.Length();
  }
  else if (PageControlFeatures->ActivePage == TabSheetAssembler)
  {
    RichEditAssembler->SetFocus();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::StringGridInstructionsKeyDown(TObject *Sender,
          WORD &Key, TShiftState Shift)
{
  switch (Key)
  {
    case VK_UP:
    {
      if (!StringGridInstructions->Row)
      {
        if (TopAddress > mMemoryStartAddress)
        {
          TopAddress--;
        }
      }
    }
    break;

    case VK_DOWN:
    {
      if (StringGridInstructions->Row == StringGridInstructions->RowCount-1)
      {
        if (TopAddress < mMaxAddress)
        {
          TopAddress++;
        }
      }
    }
    break;

    case VK_PRIOR:
    {
      if (!StringGridInstructions->Row)
      {
        if ((TopAddress-mMemoryStartAddress) > (tULong)StringGridInstructions->RowCount)
        {
          TopAddress = TopAddress - StringGridInstructions->RowCount;
        }
        else
        {
          TopAddress = mMemoryStartAddress;
        }
      }
    }
    break;

    case VK_NEXT:
    {
      if (StringGridInstructions->Row == StringGridInstructions->RowCount-1)
      {
        if ((mMaxAddress-TopAddress) > (tULong)StringGridInstructions->RowCount)
        {
          TopAddress = TopAddress + StringGridInstructions->RowCount;
        }
        else
        {
          TopAddress = mMaxAddress;
        }
      }
    }
    break;
  }

	NOT_USED(Shift);
  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonTextColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelButtonMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonTextColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonSetAddressClick(TObject *Sender)
{
  TPanel* CurrentPanel = (TPanel*)Sender;
  // Get marker number
  AnsiString PanelSetName("PanelButtonSetAddress");
  int Index = CurrentPanel->Name.SubString(PanelSetName.Length()+1, CurrentPanel->Name.Length() - PanelSetName.Length()).ToInt();
  // Fill associated panel
  AnsiString PanelMarkerName = "PanelAddress" + AnsiString(Index);
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name == PanelMarkerName)
    {
      ((TPanel*)Components[Loop])->Caption = AnsiString::IntToHex(mSelectedAddress, 4);
      // Keep marker
      mSettingsP->SetCodeExplorerMarker(Index-1, ((TPanel*)Components[Loop])->Caption);
      break;
    }
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelAddressClick(TObject *Sender)
{
  TPanel* Panel = (TPanel*)Sender;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = StrToInt("0x" + Panel->Caption);
  EditValue->Hexadecimal = PanelFormatTopAddress->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = PanelFormatTopAddress->Caption.IsEmpty() == true ? NB_DIGITS_TOP_ADDRESS_DEC : NB_DIGITS_TOP_ADDRESS_HEX;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mMaxAddress;
  if (EditValue->ShowModal() == mrOk)
  {
    Panel->Caption = AnsiString::IntToHex(EditValue->NewValue, 4);

    // Get marker number
    AnsiString PanelName("PanelAddress");
    int Index = Panel->Name.SubString(PanelName.Length()+1, Panel->Name.Length() - PanelName.Length()).ToInt();
    // Keep marker
    mSettingsP->SetCodeExplorerMarker(Index-1, Panel->Caption);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::PanelButtonGoToAddressClick(TObject *Sender)
{
  TPanel* CurrentPanel = (TPanel*)Sender;
  // Get marker number
  AnsiString PanelSetName("PanelButtonGoToAddress");
  int Index = CurrentPanel->Name.SubString(PanelSetName.Length()+1, CurrentPanel->Name.Length() - PanelSetName.Length()).ToInt();
  // Go to associated panel
  AnsiString PanelMarkerName = "PanelAddress" + AnsiString(Index);
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name == PanelMarkerName)
    {
      TopAddress = StrToInt("0x" + ((TPanel*)Components[Loop])->Caption);
      mSelectedAddress = mTopAddress;
      StringGridInstructions->SetFocus();
      StringGridInstructions->Row = 0;

      break;
    }
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormCodeExplorer::EditNoteAddressChange(TObject *Sender)
{
  TEdit* CurrentEdit = (TEdit*)Sender;
  // Get marker number
  AnsiString EditNoteAddressName("EditNoteAddress");
  int Index = CurrentEdit->Name.SubString(EditNoteAddressName.Length()+1,
                CurrentEdit->Name.Length() - EditNoteAddressName.Length()).ToInt();
  // Keep marker label
  mSettingsP->SetCodeExplorerMarkerLabel(Index-1, CurrentEdit->Text);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::PanelButtonDriveALoadClick(TObject *Sender)
{
  tDiskOperation* DiskOperationP = new tDiskOperation();
  memset(DiskOperationP, 0, sizeof(tDiskOperation));

  // Prepare File Data
  DiskOperationP->NativeCPC = FormMain->NativeCPC;
  DiskOperationP->Drive = FormMain->NativeCPC->FDC->DriveA;

  // Read catalogue
  mEmulatorP->ReadDiskCatalogue(DiskOperationP);
  mEmulatorP->ConvertFilenameToCatalogueEntry(ComboBoxCodeFilename->Text.UpperCase(), DiskOperationP);
  // Select filename
  if (mEmulatorP->SelectCatalogueEntry(DiskOperationP) == true)
  {
    // Read file header
    tFileHeader FileHeader;
    tULong FileSize = DiskOperationP->CatalogueEntrySize[DiskOperationP->CatalogueEntryIndex];
    bool FileHeaderPresent = false;
    if (mEmulatorP->ReadFileHeader(DiskOperationP, &FileHeader) == true)
    {
      FileSize = FileHeader.Length + AMSDOS_HEADER_SIZE;
      FileHeaderPresent = true;
    }

    if (FileSize > 0)
    {
	    TCursor mOldCursor = Screen->Cursor;
	    Screen->Cursor = crHourGlass;

      // Load File
      tUChar* FileDataP = new tUChar[FileSize];
      DiskOperationP->FileContentP = FileDataP;
      DiskOperationP->FileSize = FileSize;
      if (mEmulatorP->LoadDriveFile(DiskOperationP) == true)
      {
        CodeOrigin = FileHeader.DataLocation;
		    CodeSize = FileHeader.Length;

		    // Write file in RAM
		    tUChar* DataP = FileDataP;
		    if (FileHeaderPresent == true) DataP += AMSDOS_HEADER_SIZE;
		    for (int loop=0; loop < CodeSize; loop++)
		    {
		      mWriteFunction(FormMain->NativeCPC,
						             CodeOrigin + loop - mMemoryStartAddress,
                         *DataP++);
        }

        // Disassemble
		    tULong addr = CodeOrigin;
		    tULong endaddr = CodeOrigin + CodeSize;

		    RichEditAssembler->SetFocus();
		    RichEditAssembler->Lines->BeginUpdate();
		    while (addr < endaddr)
		    {
          CodeLast = addr;
			    addr = Z80dasmDecodeInstruction(addr,
					                    						&mZ80dasmParam);
			    RichEditAssembler->SelText = AnsiString(mZ80dasmParam.Instruction).LowerCase() + "\r\n";
		    }
		    RichEditAssembler->Lines->EndUpdate();
        RichEditAssembler->CaretPos = TPoint(0,0);
      }

      delete [] FileDataP;

	    Screen->Cursor = mOldCursor;
    }
  }

  delete DiskOperationP;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormCodeExplorer::PanelButtonDriveASaveClick(TObject *Sender)
{
  // Invalid code size ?
  if (mCodeSize <= 0)
  {
    Application->MessageBox(_T("Invalid size."), _T("Error"), MB_OK | MB_ICONERROR);
  }
  else
  {
    tDiskOperation* DiskOperationP = new tDiskOperation();
    memset(DiskOperationP, 0, sizeof(tDiskOperation));

    // Prepare File Data
    DiskOperationP->NativeCPC = FormMain->NativeCPC;
    DiskOperationP->Drive = FormMain->NativeCPC->FDC->DriveA;
    mEmulatorP->ConvertFilenameToCatalogueEntry(ComboBoxCodeFilename->Text.UpperCase(), DiskOperationP);
    DiskOperationP->FileStartAddress = (tULong)mCodeOrigin;
    DiskOperationP->FileSize = (tULong)(mCodeSize + AMSDOS_HEADER_SIZE);

    // Prepare file content with header
    tUChar* FileDataP = new tUChar[DiskOperationP->FileSize + AMSDOS_HEADER_SIZE];
    DiskOperationP->FileContentP = FileDataP;
    tFileHeader FileHeader;
    FileHeader.Type = AMSDOS_FILE_TYPE_BINARY;
    FileHeader.DataLength = mCodeSize;
    FileHeader.DataLocation = mCodeOrigin;
    FileHeader.Length = mCodeSize;
    tUChar* DataP = mEmulatorP->CreateFileHeader(DiskOperationP, &FileHeader);
    if (DataP)
    {
      memcpy(DataP, FormMain->NativeCPC->pbRAM + mCodeOrigin, mCodeSize);
    }

    if (mEmulatorP->AddDriveFile(DiskOperationP) == false)
    {
      Application->MessageBox(_T("Not enought free space into drive A disk."),
                              _T("Disk full"),
                              MB_OK | MB_ICONEXCLAMATION);
    }

    delete [] FileDataP;
    delete DiskOperationP;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormCodeExplorer::OnDiskEvent(tDrive* DriveP)
{
  if (DriveP->signal_content_change)
  {
    UpdateCatalogue();
  }
}
//---------------------------------------------------------------------------

