/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2018 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef TFormCodeExplorerH
#define TFormCodeExplorerH
//---------------------------------------------------------------------------
#include <System.Classes.hpp>
#include <Vcl.Controls.hpp>
#include <Vcl.StdCtrls.hpp>
#include <Vcl.Forms.hpp>
#include <Vcl.Grids.hpp>
#include <Vcl.ExtCtrls.hpp>
#include <Vcl.ComCtrls.hpp>
#include "..\Emulator\TEmulator.h"
#include "..\Settings\TSettings.h"
#include "..\..\Common\Z80DASM\z80dasm.h"
#include <Vcl.Dialogs.hpp>
#include <Vcl.Menus.hpp>


typedef void (*TCodeExplorerWriteFunction)(tNativeCPC* NativeCPC, tULong addr, tUChar val);


//---------------------------------------------------------------------------
class TFormCodeExplorer : public TForm
{
__published:	// Composants grs par l'EDI
  TLabel *LabelTopAddress;
  TPanel *PanelFormatTopAddress;
  TPanel *PanelTopAddress;
  TTabControl *TabControlMemoryContent;
  TStringGrid *StringGridInstructions;
  TScrollBar *ScrollBarAddress;
  TOpenDialog *OpenDialogAssembler;
  TSaveDialog *SaveDialogAssembler;
  TPageControl *PageControlFeatures;
  TTabSheet *TabSheetPatch;
  TPanel *PanelPatchMachine;
  TTabSheet *TabSheetAssembler;
  TLabel *LabelCodeOrigin;
  TPanel *PanelFormatCodeOrigin;
  TPanel *PanelCodeOrigin;
  TLabel *LabelCodeSize;
  TPanel *PanelFormatCodeSize;
  TPanel *PanelCodeSize;
  TPopupMenu *PopupMenuAssembler;
  TMenuItem *CopyMenu;
  TMenuItem *ClearMenu;
  TMenuItem *PasteMenu;
  TMenuItem *SelectAllMenu;
  TEdit *EditPatchAssembly;
  TTabSheet *TabSheetMarkers;
  TPanel *PanelButtonSetAddress1;
  TPanel *PanelAddress1;
  TPanel *PanelButtonGoToAddress1;
  TPanel *PanelButtonRefresh;
  TPanel *PanelButtonRegPC;
  TPanel *PanelButtonPatchUpdate;
  TPanel *PanelButtonSetAddress2;
  TPanel *PanelAddress2;
  TPanel *PanelButtonGoToAddress2;
  TEdit *EditNoteAddress1;
  TEdit *EditNoteAddress2;
  TPanel *PanelButtonSetCodeOrigin;
  TPanel *PanelButtonGoToCodeOrigin;
  TPanel *PanelButtonCopy;
  TPanel *PanelButtonAssemblerUpdate;
  TPanel *PanelButtonAssemblerLoad;
  TPanel *PanelButtonAssemblerSave;
  TEdit *EditNoteAddress3;
  TPanel *PanelButtonSetAddress3;
  TPanel *PanelAddress3;
  TPanel *PanelButtonGoToAddress3;
  TEdit *EditNoteAddress4;
  TPanel *PanelButtonSetAddress4;
  TPanel *PanelAddress4;
  TPanel *PanelButtonGoToAddress4;
  TRichEdit *RichEditAssembler;
  TComboBox *ComboBoxCodeFilename;
  TLabel *LabelFile;
  TPanel *PanelButtonDriveASave;
  TLabel *LabelCodeLast;
  TPanel *PanelButtonSetCodeLast;
  TPanel *PanelFormatCodeLast;
  TPanel *PanelCodeLast;
  TPanel *PanelButtonGoToCodeLast;
  TPanel *PanelButtonDriveALoad;
  void __fastcall FormDestroy(TObject *Sender);
  void __fastcall FormShow(TObject *Sender);
  void __fastcall FormHide(TObject *Sender);
  void __fastcall FormResize(TObject *Sender);
  void __fastcall StringGridInstructionsDrawCell(TObject *Sender, int ACol, int ARow,
          TRect &Rect, TGridDrawState State);
  void __fastcall PanelTopAddressClick(TObject *Sender);
  void __fastcall ScrollBarAddressChange(TObject *Sender);
  void __fastcall PanelFormatTopAddressClick(TObject *Sender);
  void __fastcall StringGridInstructionsMouseWheelDown(TObject *Sender, TShiftState Shift,
          TPoint &MousePos, bool &Handled);
  void __fastcall StringGridInstructionsMouseWheelUp(TObject *Sender, TShiftState Shift,
          TPoint &MousePos, bool &Handled);
  void __fastcall StringGridInstructionsDblClick(TObject *Sender);
  void __fastcall PanelFormatCodeOriginClick(TObject *Sender);
  void __fastcall PanelCodeOriginClick(TObject *Sender);
  void __fastcall StringGridInstructionsClick(TObject *Sender);
  void __fastcall EditPatchAssemblyChange(TObject *Sender);
  void __fastcall TabControlMemoryContentDrawTab(TCustomTabControl *Control, int TabIndex,
          const TRect &Rect, bool Active);
  void __fastcall TabControlMemoryContentChange(TObject *Sender);
  void __fastcall PageControlFeaturesDrawTab(TCustomTabControl *Control, int TabIndex,
          const TRect &Rect, bool Active);
  void __fastcall PanelFormatCodeSizeClick(TObject *Sender);
  void __fastcall ClearMenuClick(TObject *Sender);
  void __fastcall SelectAllMenuClick(TObject *Sender);
  void __fastcall CopyMenuClick(TObject *Sender);
  void __fastcall PasteMenuClick(TObject *Sender);
  void __fastcall PageControlFeaturesChange(TObject *Sender);
  void __fastcall StringGridInstructionsKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
  void __fastcall PanelButtonMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
  void __fastcall PanelButtonMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y);
  void __fastcall PanelButtonSetAddressClick(TObject *Sender);
  void __fastcall PanelAddressClick(TObject *Sender);
  void __fastcall PanelButtonGoToAddressClick(TObject *Sender);
  void __fastcall PanelButtonRefreshClick(TObject *Sender);
  void __fastcall PanelButtonRegPCClick(TObject *Sender);
  void __fastcall PanelButtonPatchUpdateClick(TObject *Sender);
  void __fastcall PanelButtonSetCodeOriginClick(TObject *Sender);
  void __fastcall PanelButtonGoToCodeOriginClick(TObject *Sender);
  void __fastcall PanelButtonCopyClick(TObject *Sender);
  void __fastcall PanelButtonAssemblerUpdateClick(TObject *Sender);
  void __fastcall PanelButtonAssemblerLoadClick(TObject *Sender);
  void __fastcall PanelButtonAssemblerSaveClick(TObject *Sender);
  void __fastcall EditNoteAddressChange(TObject *Sender);
  void __fastcall RichEditAssemblerKeyDown(TObject *Sender, WORD &Key, TShiftState Shift);
  void __fastcall PanelButtonDriveASaveClick(TObject *Sender);
  void __fastcall PanelButtonSetCodeLastClick(TObject *Sender);
  void __fastcall PanelFormatCodeLastClick(TObject *Sender);
  void __fastcall PanelCodeLastClick(TObject *Sender);
  void __fastcall PanelButtonGoToCodeLastClick(TObject *Sender);
  void __fastcall PanelButtonDriveALoadClick(TObject *Sender);





private:	// Dclarations utilisateur
  //
  // Members
  //
  TEmulator* mEmulatorP;
  TSettings* mSettingsP;
  bool mPaused;
  tULong mTopAddress;
  tULong mMaxAddress;
  tLong mCodeOrigin;
  tLong mCodeLast;
  tLong mCodeSize;
  tULong mSelectedAddress;
  tULong mMemoryStartAddress;
  tULong mMemorySize;
  TZ80dasmParam mZ80dasmParam;
  TCodeExplorerWriteFunction mWriteFunction;
  AnsiString mAssemblerBASICFilename;

  //
  // Methods
  //
  AnsiString __fastcall GetTitle(void);
  void __fastcall WMMove(TMessage &Message);
  inline void __fastcall SetEmulator(TEmulator* Value) { mEmulatorP=Value; };
  void __fastcall SetSettings(TSettings* Value);
  void __fastcall UpdateInstructions(tNativeCPC* NativeCPC);
  void __fastcall UpdatePatch(tNativeCPC* NativeCPC);
  void __fastcall UpdateCatalogue(void);
  inline tULong __fastcall GetTopAddress(void) { return mTopAddress; };
  void __fastcall SetTopAddress(tULong Value);
  inline tLong __fastcall GetCodeOrigin(void) { return mCodeOrigin; };
  void __fastcall SetCodeOrigin(tLong Value);
  inline tLong __fastcall GetCodeLast(void) { return mCodeLast; };
  void __fastcall SetCodeLast(tLong Value);
  inline tLong __fastcall GetCodeSize(void) { return mCodeSize; };
  void __fastcall SetCodeSize(tLong Value);
  void __fastcall SetMemoryIndex(int Value);
  void __fastcall OnDiskEvent(tDrive* DriveP);

  //
  // Private Properties
  //
  __property AnsiString Title = { read=GetTitle };
  __property tULong TopAddress = { read=GetTopAddress, write=SetTopAddress };
  __property tLong CodeOrigin = { read=GetCodeOrigin, write=SetCodeOrigin };
  __property tLong CodeLast = { read=GetCodeLast, write=SetCodeLast };
  __property tLong CodeSize = { read=GetCodeSize, write=SetCodeSize };
  __property int MemoryIndex = { write=SetMemoryIndex };

public:		// Dclarations utilisateur
  //
  // Public Properties
  //
  __property TEmulator* Emulator = { write=SetEmulator };
  __property TSettings* Settings = { write=SetSettings };

  //
  // API
  //
  __fastcall TFormCodeExplorer(TComponent* Owner);

#pragma option push -vi-
BEGIN_MESSAGE_MAP
  MESSAGE_HANDLER(WM_MOVE, TMessage, WMMove);
END_MESSAGE_MAP(TForm)
#pragma option pop
};
//---------------------------------------------------------------------------
extern PACKAGE TFormCodeExplorer *FormCodeExplorer;
//---------------------------------------------------------------------------
#endif
