/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2015 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <vcl.h>
#pragma hdrstop
#include "TFormDebugger.h"
#include "TFormMain.h"


//=================================
// Options

//=================================


//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

static const AnsiString cTitle = "Execute";
static const AnsiString cTitlePaused = " - [PAUSED]";
static const AnsiString cTitlePlaying = " - [Playing]";
static const AnsiString cTitleRunning = " - [Running]";

TFormDebugger *FormDebugger;
//---------------------------------------------------------------------------
__fastcall TFormDebugger::TFormDebugger(TComponent* Owner)
        : TForm(Owner)
{
  mSettingsP = NULL;

  // Apply theme
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::FormShow(TObject *Sender)
{
  if (FormMain == NULL)
  {
    Close();
    return;
  }

  mTimerLastPause = 0;
  LabelTimeFromResetS->Caption = "";
  LabelTimeFromResetUS->Caption = "";
  LabelTimeFromLastPauseMS->Caption = "";
  LabelTimeFromLastPauseUS->Caption = "";

  mPaused = false;
  mPauseRequested = false;
  mStepByStep = false;
  mRunTillVBL = false;
  mRunTillHBL = false;

  FormMain->SubscribeToCPCEvents(OnCPCEvent);

  UpdateControls(FormMain->EmulatorP->Paused,
                 FormMain->EmulatorP->StepByStepActive);
  Caption = Title;

  if (mSettingsP)
  {
    mSettingsP->DebuggerFormVisible = true;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::FormHide(TObject *Sender)
{
  // Unsubscribe to step by step event
  if (FormMain != NULL)
  {
    FormMain->UnsubscribeToCPCEvents(OnCPCEvent);
  }

  if (mSettingsP)
  {
    mSettingsP->DebuggerFormVisible = false;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


AnsiString __fastcall TFormDebugger::GetTitle(void)
{
  AnsiString Title = cTitle;
  if (mPaused == true) { Title += cTitlePaused; }
  else if (mStepByStep == true) { Title += cTitlePlaying; }
  else { Title += cTitleRunning; }
  
 return Title;
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::SetSettings(TSettings* Value)
{
  mSettingsP = Value;

  if (    (mSettingsP->DebuggerFormLeft != 0)
       || (mSettingsP->DebuggerFormTop != 0)
       /*|| (mSettings->DebuggerFormWidth != 0)
       || (mSettings->DebuggerFormHeight != 0)*/ )
  {
    Position = poDesigned;
    Left = mSettingsP->DebuggerFormLeft;
    Top = mSettingsP->DebuggerFormTop;
    /*Width = mSettings->DebuggerFormWidth;
    Height = mSettings->DebuggerFormHeight;*/
  }

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  Font->Color = mSettingsP->ThemeText1Color;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("PanelButton") == 1)
    {
      ((TPanel*)Components[Loop])->Color = mSettingsP->ThemeButtonColor;
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeButtonTextColor;
    }
  }

  // Refresh display
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (((TControl*)Components[Loop])->Visible == true)
    {
      ((TControl*)Components[Loop])->Invalidate();
    }
  }

  if (mSettingsP->HideAll == false)
  {
    Visible = mSettingsP->DebuggerFormVisible;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormDebugger::FormResize(TObject *Sender)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->DebuggerFormWidth = Width;
    mSettingsP->DebuggerFormHeight = Height;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormDebugger::WMMove(TMessage &Message)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->DebuggerFormLeft = Left;
    mSettingsP->DebuggerFormTop = Top;
  }

	NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::PanelButtonStepClick(TObject *Sender)
{
  mRunTillVBL = false;
  mRunTillHBL = false;

  mPauseRequested = true;
  FormMain->ResumeStepByStep(OnCPCEvent);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormDebugger::PanelButtonPlayClick(TObject *Sender)
{
  mRunTillVBL = false;
  mRunTillHBL = false;

  if (PanelButtonPlay->Caption == "Play")
  {
    FormMain->ResumeStepByStep(OnCPCEvent);
  }
  else
  {
    mPauseRequested = true;
    FormMain->Pause(true);
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormDebugger::PanelButtonRunClick(TObject *Sender)
{
  mRunTillVBL = false;
  mRunTillHBL = false;

  if (PanelButtonRun->Caption == "Run")
  {
    FormMain->StopStepByStep(false);
  }
  else
  {
    mPauseRequested = true;
    FormMain->Pause(true);
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::PanelButtonPlayTillVBLClick(TObject *Sender)
{
  mRunTillVBL = true;
  mRunTillHBL = false;

  mPreviousVBL = Engine_IsVideoVBL(FormMain->EmulatorP->NativeCPC);
  FormMain->ResumeStepByStep(OnCPCEvent);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormDebugger::PanelButtonPlayTillHBLClick(TObject *Sender)
{
  mRunTillVBL = false;
  mRunTillHBL = true;

  mPreviousHBL = Engine_IsVideoHBL(FormMain->EmulatorP->NativeCPC);
  FormMain->ResumeStepByStep(OnCPCEvent);

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::UpdateControls(bool Pause, bool StepByStep)
{
  if (StepByStep == true)
  {
    PanelButtonStep->Visible = Pause;
    PanelButtonPlay->Visible = true;
    PanelButtonPlay->Caption = (Pause == false) ? "Pause" : "Play";
    PanelButtonRun->Caption = "Run";
  }
  else
  {
    PanelButtonStep->Visible = Pause;
    PanelButtonPlay->Visible = Pause;
    PanelButtonPlay->Caption = "Play";
    PanelButtonRun->Caption = (Pause == false) ? "Pause" : "Run";
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::UpdateTimes(void)
{
  // Display times
  tULong TimerFromReset = FormMain->EmulatorP->NativeCPC->timer_from_reset;

  if (mTimerLastPause != TimerFromReset)
  {
    LabelTimeFromResetS->Caption = ((TimerFromReset / 4) / 1000000);
    LabelTimeFromResetUS->Caption = (TimerFromReset / 4);
    
    if (mTimerLastPause > TimerFromReset)
    {
      LabelTimeFromLastPauseMS->Caption = "";
      LabelTimeFromLastPauseUS->Caption = "";
      mTimerLastPause = 0;
    }

    if (mTimerLastPause != 0)
    {
      LabelTimeFromLastPauseMS->Caption = (((TimerFromReset - mTimerLastPause) / 4) / 1000);
      LabelTimeFromLastPauseUS->Caption = ((TimerFromReset - mTimerLastPause) / 4);
    }
    mTimerLastPause = TimerFromReset;
  }
}
//---------------------------------------------------------------------------


bool __fastcall TFormDebugger::OnCPCEvent(tULong Condition, tNativeCPC* NativeCPC)
{
  bool Pause = NativeCPC->paused;
  bool StepByStep = NativeCPC->step_by_step;

  if (mPauseRequested == true)
  {
    Pause = true;
    mPauseRequested = false;
  }

  if ( (Pause != mPaused) || (StepByStep != mStepByStep) )
  {
    UpdateControls(Pause, StepByStep);

    mPaused = Pause;
    mStepByStep = StepByStep;
    Caption = Title;
  }

  if ( (Condition & EC_END_INSTRUCTION) == 0)
    return Pause;

  // Check and Update VBL
  if (mRunTillVBL == true)
  {
    bool CurrentVBL = Engine_IsVideoVBL(NativeCPC);
    if (mPreviousVBL != CurrentVBL)
    {
      if (CurrentVBL == true)
      {
        mRunTillVBL = false;
        Pause = true;
      }

      mPreviousVBL = CurrentVBL;
    }
  }

  // Check and Update HBL
  if (mRunTillHBL == true)
  {
    bool CurrentHBL = Engine_IsVideoHBL(NativeCPC);
    if (mPreviousHBL != CurrentHBL)
    {
      if (CurrentHBL == true)
      {
        mRunTillHBL = false;
        Pause = true;
      }

      mPreviousHBL = CurrentHBL;
    }
  }

  if (Pause == true)
  {
    UpdateTimes();
  }

	NOT_USED(Condition);

	return Pause;
}
//---------------------------------------------------------------------------


void __fastcall TFormDebugger::PanelButtonMouseDown(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonTextColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

void __fastcall TFormDebugger::PanelButtonMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonTextColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

