/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2016 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <vcl.h>
#include <FileCtrl.hpp>
#include <tchar.h>
#pragma hdrstop
#include "TFormEmulatorSettings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"


#define SETTINGS_FILTER             "CaPriCe Settings (.ini)|*.ini|"\
                                    "All files|*.*"

#define SETTINGS_INI_EXT            ".ini"


typedef struct
{
  const char* Name;
  tUChar CPCModel;
} tDeviceInfo;


#define NB_DEVICES    8
static const tDeviceInfo Devices[NB_DEVICES]=
{
  { "CPC 464 UK", CPC_MODEL_464_UK },
  { "CPC 464 FR", CPC_MODEL_464_FR },
  { "CPC 464 SP", CPC_MODEL_464_SP },
  { "CPC 664 UK FDD", CPC_MODEL_664_UK },
  { "CPC 664 UK LDD", CPC_MODEL_664_FR },
  { "CPC 6128 UK", CPC_MODEL_6128_UK },
  { "CPC 6128 FR", CPC_MODEL_6128_FR },
  { "CPC 6128 SP", CPC_MODEL_6128_SP }
};


TFormEmulatorSettings *FormEmulatorSettings;
//---------------------------------------------------------------------------
__fastcall TFormEmulatorSettings::TFormEmulatorSettings(TComponent* Owner)
  : TForm(Owner)
{
  mSettingsP = NULL;
  mAvailableROMList = NULL;

#ifndef USE_SILICON_DISC
  CheckBoxUse256kSiliconDisc->Visible = false;
#endif /* USE_SILICON_DISC */

#ifndef USE_RAM_4MB
  CheckBoxUse4MMemoryExtension->Visible = false;
#endif /* USE_RAM_4MB */

#ifndef ENABLE_TMPI_SPEECH
  CheckBoxUseTechniMusique->Visible = false;
#endif /* ENABLE_TMPI_SPEECH */

#ifndef ENABLE_PLAYCITY
  CheckBoxUsePlayCity->Visible = false;
#endif /* ENABLE_PLAYCITY */

#ifndef ENABLE_DIGIBLASTER
  CheckBoxUseDigiblaster->Visible = false;
#endif /* ENABLE_DIGIBLASTER */

#ifndef ENABLE_MULTIFACE2
  CheckBoxUseMultiface2->Visible = false;
#endif /* ENABLE_MULTIFACE2 */

  // Populate devices
  ComboBoxDevices->Items->Clear();
  for (int Index=0; Index < NB_DEVICES; Index++)
  {
    ComboBoxDevices->Items->Add(Devices[Index].Name);
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::FormShow(TObject *Sender)
{
	if (mSettingsP == NULL)
  {
    ModalResult = mrCancel;
    return;
  }

  ShowHint = mSettingsP->ShowHints;

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  StringGridROMMapping->FixedColor = Color;
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;
  Font->Color = mSettingsP->ThemeText1Color;
  ComboBoxDevices->Font->Color = clBlack;
  ComboBoxBrands->Font->Color = clBlack;
  ComboBoxCRTCType->Font->Color = clBlack;
  ComboBoxAvailableROM->Font->Color = clBlack;
  ComboBoxMF2Rom->Font->Color = clBlack;
  ComboBoxSettings->Font->Color = clBlack;
  GroupBoxMachine->Font->Color = mSettingsP->ThemeText2Color;
  LabelDevice->Font->Color = mSettingsP->ThemeText1Color;
  LabelBrand->Font->Color = mSettingsP->ThemeText1Color;
  LabelCRTCType->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUseFloppyDrive->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUseFastFDC->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUse64kMemoryExtension->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUse256kMemoryExtension->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUse256kSiliconDisc->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUse4MMemoryExtension->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUseTechniMusique->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUsePlayCity->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUseDigiblaster->Font->Color = mSettingsP->ThemeText1Color;
  CheckBoxUseMultiface2->Font->Color = mSettingsP->ThemeText1Color;
  GroupBoxROMMapping->Font->Color = mSettingsP->ThemeText2Color;
  LabelSelectROM->Font->Color = mSettingsP->ThemeText1Color;
  GroupBoxMultiface2->Font->Color = mSettingsP->ThemeText2Color;
  LabelMF2SelectROM->Font->Color = mSettingsP->ThemeText1Color;
  GroupBoxSettings->Font->Color = mSettingsP->ThemeText2Color;
  PanelSettingsSave->Color = mSettingsP->ThemeButtonColor;
  PanelSettingsSave->Font->Color = mSettingsP->ThemeButtonTextColor;
  PanelApply->Color = mSettingsP->ThemeButtonColor;
  PanelApply->Font->Color = mSettingsP->ThemeButtonTextColor;
  PanelCancel->Color = mSettingsP->ThemeButtonColor;
  PanelCancel->Font->Color = mSettingsP->ThemeButtonTextColor;

  //
  // List available ROMs
  //
  mAvailableROMList = new TStringList;
  AnsiString ROMPath = ExpandFileName(mSettingsP->RomsDirectory);
  if (DirectoryExists(ROMPath))
  {
    TSearchRec F;
    if (FindFirst(ROMPath + "*.rom", faAnyFile, F) == 0)
    {
      int findresult;
      do
      {
        mAvailableROMList->Add(F.Name);
        findresult = FindNext(F);
      }
      while (findresult == 0);
    }
  }

  //
  // Multiface 2 Rom
  //
  ComboBoxMF2Rom->Clear();
  ComboBoxMF2Rom->Items->Add(DefaultMultiface2RomName);
  // Add list of user ROMs
  for (int Loop=0; Loop < mAvailableROMList->Count; Loop++)
  {
    ComboBoxMF2Rom->Items->Add(mAvailableROMList->Strings[Loop]);
  }

  //
  // Roms mapping
  //
  StringGridROMMapping->RowCount = NB_ROMS;
  StringGridROMMapping->ColWidths[0] = 100;
  StringGridROMMapping->ColWidths[1] = StringGridROMMapping->Width - StringGridROMMapping->ColWidths[0];
  StringGridROMMapping->Cells[0][0] = "LowerROM";
  for (int loop=0; loop < NB_UPPER_ROMS; loop++)
  {
    AnsiString Header = "UpperROM ";
    if (loop < 10) Header += " ";
    StringGridROMMapping->Cells[0][loop+1] = Header + AnsiString(loop);
  }

  // List available Settings
  UpdateAvailableSettings("");

  // Display settings
  DisplaySettings(mSettingsP);

  ActiveControl = StringGridROMMapping;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::FormHide(TObject *Sender)
{
	delete mAvailableROMList;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::PanelApplyClick(TObject *Sender)
{
  // Apply new settings
  StoreSettings(mSettingsP);

  ModalResult = mrOk;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::PanelCancelClick(TObject *Sender)
{
  ModalResult = mrCancel;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::FormResize(TObject *Sender)
{
	StringGridROMMapping->ColWidths[1] = StringGridROMMapping->Width - StringGridROMMapping->ColWidths[0];

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::StringGridROMMappingMouseDown(
			TObject *Sender, TMouseButton Button, TShiftState Shift, int X,
      int Y)
{
	// Get selected row
  int SelectedCol;
  int SelectedRow;
  StringGridROMMapping->MouseToCell(X, Y, SelectedCol, SelectedRow);
  if (SelectedRow != -1)
  {
    StringGridROMMapping->Row = SelectedRow;
    UpdateAvailableROMList(SelectedRow);
  }

  NOT_USED(Sender);
  NOT_USED(Button);
  NOT_USED(Shift);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::FillROMMapping(TSettings* SettingsP)
{
  //
  // Lower ROM
  //
  if (SettingsP->LowerROMFilename.IsEmpty() == false)
  {
    StringGridROMMapping->Cells[1][0] = SettingsP->LowerROMFilename;
  }
  else
  {
    StringGridROMMapping->Cells[1][0] = DefaultOSRomName;
  }

  //
  // Upper ROMs
  //
  for (int loop=0; loop < NB_UPPER_ROMS; loop++)
  {
    if (SettingsP->UpperROMFilename[loop].IsEmpty() == false)
    {
      // Default ROM to remove !!
      if (SettingsP->UpperROMFilename[loop] == EmptyRomName)
      {
        StringGridROMMapping->Cells[1][loop+1] = "";
      }
      else
      {
        StringGridROMMapping->Cells[1][loop+1] = SettingsP->UpperROMFilename[loop];
      }
    }
    else
    {
      // BASIC ROM
      if (loop == 0)
      {
        StringGridROMMapping->Cells[1][loop+1] = DefaultBASICRomName;
      }
      // AMSDOS ROM
      else if (loop == 7)
      {
        StringGridROMMapping->Cells[1][loop+1] = DefaultAMSDOSRomName;
      }
      // Other ROMs
      else
      {
        StringGridROMMapping->Cells[1][loop+1] = "";
      }
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::UpdateAvailableROMList(int Index)
{
  if (Index < 0) return;

  ComboBoxAvailableROM->Clear();

  //
  // Add Default ROMs
  //
  if (Index == 0) // Lower ROM
  {
    ComboBoxAvailableROM->Items->Add(DefaultOSRomName);
  }
  else if (Index == 1) // Upper ROM 0
  {
    ComboBoxAvailableROM->Items->Add(DefaultBASICRomName);
  }
  else
  {
    // Add Empty for all others
    ComboBoxAvailableROM->Items->Add(EmptyRomName);
    ComboBoxAvailableROM->Items->Add(DefaultAMSDOSRomName);
#ifdef USE_SILICON_DISC
    ComboBoxAvailableROM->Items->Add(DefaultSiliconDiscRomName);
#endif /* USE_SILICON_DISC */
  }

  // Add list of user ROMs
  for (int Loop=0; Loop < mAvailableROMList->Count; Loop++)
  {
    ComboBoxAvailableROM->Items->Add(mAvailableROMList->Strings[Loop]);
  }

  // Upper ROM ?
  if (Index > 0)
  {
    // Add temporary Upper ROM
    AnsiString UpperROM = StringGridROMMapping->Cells[1][Index-1];
    if (UpperROM.IsEmpty() == false)
    {
      // Temporary ROM ?
      if (ComboBoxAvailableROM->Items->IndexOf(UpperROM) == -1)
      {
        ComboBoxAvailableROM->Items->Add(UpperROM);
      }
    }
  }

  //
  // Select current ROM
  //
  AnsiString CurrentROM = StringGridROMMapping->Cells[1][Index];
  if (CurrentROM.IsEmpty() == true)
  {
    CurrentROM = EmptyRomName;
  }
  ComboBoxAvailableROM->ItemIndex = ComboBoxAvailableROM->Items->IndexOf(CurrentROM);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::UpdateAvailableSettings(AnsiString SelectedFilename)
{
  AnsiString SettingsPath = ExpandFileName(mSettingsP->SettingsDirectory);
  AnsiString SelectedSettings = "";
  ComboBoxSettings->Clear();
  if (DirectoryExists(SettingsPath))
  {
    TSearchRec F;
    if (FindFirst(SettingsPath + "*.ini", faAnyFile, F) == 0)
    {
      int findresult;
      do
      {
        AnsiString Settings = AnsiString(F.Name).SubString(1,
                                                        AnsiString(F.Name).Pos(SETTINGS_INI_EXT) - 1);
        ComboBoxSettings->Items->Add(Settings);
        if (F.Name == SelectedFilename)
        {
          SelectedSettings = Settings;
        }
        findresult = FindNext(F);
      }
      while (findresult == 0);
    }

    // Select settings
    if (SelectedSettings.IsEmpty() == false)
    {
      ComboBoxSettings->ItemIndex = ComboBoxSettings->Items->IndexOf(SelectedSettings);
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::ComboBoxAvailableROMChange(
      TObject *Sender)
{
AnsiString NewROM = ComboBoxAvailableROM->Text;

	if (NewROM == EmptyRomName)
	{
		NewROM = "";
	}

	StringGridROMMapping->Cells[1][StringGridROMMapping->Row] = NewROM;
	ActiveControl = StringGridROMMapping;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::ComboBoxAvailableROMDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxAvailableROM->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxAvailableROM->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxAvailableROM->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Index);
  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::StringGridROMMappingSelectCell(
			TObject *Sender, int ACol, int ARow, bool &CanSelect)
{
	UpdateAvailableROMList(ARow);

  NOT_USED(Sender);
  NOT_USED(ACol);
  NOT_USED(CanSelect);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::StringGridROMMappingKeyDown(
			TObject *Sender, WORD &Key, TShiftState Shift)
{
	if (StringGridROMMapping->Focused() == true)
  {
    // Select first ROM of the available ROM list
    if ( (Key == VK_DELETE) || (Key == VK_BACK) )
    {
      ComboBoxAvailableROM->ItemIndex = 0;
      ComboBoxAvailableROMChange(Sender);
    }
  }

  NOT_USED(Shift);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::StringGridROMMappingDrawCell(TObject *Sender,
          int ACol, int ARow, TRect &Rect, TGridDrawState State)
{
  // Fixed column ?
  if (ACol == 0)
  {
    StringGridROMMapping->Canvas->Brush->Color = Color;
    StringGridROMMapping->Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }
  else
  {
    if (State.Contains(gdSelected))
    {
      StringGridROMMapping->Canvas->Brush->Color = clHighlight;
      StringGridROMMapping->Canvas->Font->Color = clWhite;
    }
    else
    {
      StringGridROMMapping->Canvas->Brush->Color = Color;
      StringGridROMMapping->Canvas->Font->Color = Font->Color;
    }
  }

  // Draw cell
  StringGridROMMapping->Canvas->FillRect(Rect);
  int TextHeight = StringGridROMMapping->Canvas->TextHeight(StringGridROMMapping->Cells[ACol][ARow]);
  int TextPos = (StringGridROMMapping->RowHeights[ARow] - TextHeight) / 2;
  StringGridROMMapping->Canvas->TextOut(Rect.Left+5,
                                        Rect.Top+TextPos,
                                        StringGridROMMapping->Cells[ACol][ARow]);
  if (State.Contains(gdFocused))
  {
    StringGridROMMapping->Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(ARow);
  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::ComboBoxDevicesChange(TObject *Sender)
{
  // Floppy drive use
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_464_UK:
    case CPC_MODEL_464_FR:
    case CPC_MODEL_464_SP:
    {
      CheckBoxUseFloppyDrive->Checked = false;
    }
    break;

    default:
    {
      CheckBoxUseFloppyDrive->Checked = true;
    }
  }

  // 64Ko expansion
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_464_UK:
    case CPC_MODEL_464_FR:
    case CPC_MODEL_464_SP:
    case CPC_MODEL_664_UK:
    case CPC_MODEL_664_FR:
    {
      CheckBoxUse64kMemoryExtension->Checked = false;
      CheckBoxUse64kMemoryExtension->Enabled = true;
    }
    break;

    default:
    {
      CheckBoxUse64kMemoryExtension->Checked = true;
      CheckBoxUse64kMemoryExtension->Enabled = false;
    }
  }

  // Default AMSDOS
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_464_UK:
    case CPC_MODEL_464_FR:
    case CPC_MODEL_464_SP:
    {
      // Remove Default AMSDOS
      if (StringGridROMMapping->Cells[1][8] == DefaultAMSDOSRomName)
      {
        StringGridROMMapping->Cells[1][8] = "";
      }
    }
    break;

    default:
    {
      // Add Default AMSDOS
      if (StringGridROMMapping->Cells[1][8].IsEmpty() == true)
      {
        StringGridROMMapping->Cells[1][8] = DefaultAMSDOSRomName;
      }
    }
  }

  // Brand name
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_464_SP:
    {
      // Amstrad ONLY
      ComboBoxBrands->ItemIndex = 0;
      ComboBoxBrands->Enabled = false;
    }
    break;

    default:
    {
      ComboBoxBrands->Enabled = true;
    }
  }

  // CRTC Type
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_664_UK:
    case CPC_MODEL_664_FR:
    {
      // CRTC0 Only
      ComboBoxCRTCType->ItemIndex = 0;
      ComboBoxCRTCType->Enabled = false;
    }
    break;

    default:
    {
      ComboBoxCRTCType->Enabled = true;
    }
  }

#ifdef USE_RAM_4MB
  if (CheckBoxUse4MMemoryExtension->Checked == true)
  {
    CheckBoxUse64kMemoryExtension->Enabled = false;
    CheckBoxUse256kMemoryExtension->Enabled = false;
#ifdef USE_SILICON_DISC
    CheckBoxUse256kSiliconDisc->Enabled = false;
#endif /* USE_SILICON_DISC */
  }
#endif /* USE_RAM_4MB */

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::ComboBoxDevicesDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxDevices->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    switch (Devices[Index].CPCModel)
    {
      case CPC_MODEL_464_UK:
      case CPC_MODEL_464_FR:
      case CPC_MODEL_464_SP:
      {
        Canvas->Font->Color = clRed;
      }
      break;

      case CPC_MODEL_664_UK:
      case CPC_MODEL_664_FR:
      {
        Canvas->Font->Color = clGreen;
      }
      break;

      case CPC_MODEL_6128_UK:
      case CPC_MODEL_6128_FR:
      case CPC_MODEL_6128_SP:
      {
        Canvas->Font->Color = clBlue;
      }
      break;

      default:
      {
        Canvas->Font->Color = clBlack;
      }
    }
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxDevices->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxDevices->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::ComboBoxBrandsDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxBrands->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxBrands->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxBrands->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Index);
  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::CheckBoxUseFloppyDriveClick(TObject *Sender)

{
  // Add Default AMSDOS
  if (StringGridROMMapping->Cells[1][8].IsEmpty() == true)
  {
    StringGridROMMapping->Cells[1][8] = DefaultAMSDOSRomName;
  }

  if (CheckBoxUseFloppyDrive->Checked == true)
  {
    CheckBoxUseFastFDC->Enabled = true;
  }
  else
  {
    CheckBoxUseFastFDC->Checked = false;
    CheckBoxUseFastFDC->Enabled = false;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::ComboBoxCRTCTypeChange(
      TObject *Sender)
{
  // CRTC 1 choosen
  if (ComboBoxCRTCType->ItemIndex == 1)
  {
    Application->MessageBox(_T("CRTC 1 is still under development and not fully functional.\r\nMany features like lightguns are missing."), _T("Under development feature"), MB_OK);
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::ComboBoxCRTCTypeDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxCRTCType->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxCRTCType->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxCRTCType->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::CheckBoxUse256kSiliconDiscClick(TObject *Sender)

{
  if (CheckBoxUse256kSiliconDisc->Focused() == true)
  {
    UpdateSiliconDisc();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::UpdateSiliconDisc(void)
{
  if (CheckBoxUse256kSiliconDisc->Checked == true)
  {
    int loop;
    // Find Silicon Disc ROM
    for (loop=1; loop < NB_UPPER_ROMS; loop++)
    {
      if (loop == 7) continue; // Already treated

      AnsiString ROM = StringGridROMMapping->Cells[1][loop+1];
      if (ROM == DefaultSiliconDiscRomName)
      {
        break;
      }
    }

    // Find empty ROM location for SiliconDisk
    if (loop == NB_UPPER_ROMS)
    {
      for (loop=1; loop < NB_UPPER_ROMS; loop++)
      {
        if (loop == 7) continue; // Already treated

        AnsiString ROM = StringGridROMMapping->Cells[1][loop+1];
        if (ROM == DefaultSiliconDiscRomName)
        {
          break;
        }
        else if (ROM.IsEmpty() == true)
        {
          StringGridROMMapping->Cells[1][loop+1] = DefaultSiliconDiscRomName;
          break;
        }
      }
    }

    // No empty location
    if (loop == NB_UPPER_ROMS)
    {
      Application->MessageBox(_T("No available ROM location for Silicon Disc."), _T("ROM"), MB_OK);
      CheckBoxUse256kSiliconDisc->Checked = false;
    }
  }
  else
  {
    // Remove Silicon Disc ROM
    for (int loop=1; loop < NB_UPPER_ROMS; loop++)
    {
      if (loop == 7) continue; // Already treated

      AnsiString ROM = StringGridROMMapping->Cells[1][loop+1];
      if (ROM == DefaultSiliconDiscRomName)
      {
        StringGridROMMapping->Cells[1][loop+1] = "";
        break;
      }
    }
  }

  UpdateAvailableROMList(StringGridROMMapping->Row);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::CheckBoxUse4MMemoryExtensionClick(
      TObject *Sender)
{
#ifdef USE_RAM_4MB
  if (CheckBoxUse4MMemoryExtension->Checked == true)
  {
    CheckBoxUse64kMemoryExtension->Checked = false;
    CheckBoxUse64kMemoryExtension->Enabled = false;
    CheckBoxUse256kMemoryExtension->Checked = false;
    CheckBoxUse256kMemoryExtension->Enabled = false;
#ifdef USE_SILICON_DISC
    CheckBoxUse256kSiliconDisc->Checked = false;
    CheckBoxUse256kSiliconDisc->Enabled = false;
#endif /* USE_SILICON_DISC */
  }
  else
  {
    CheckBoxUse256kMemoryExtension->Checked = false;
    CheckBoxUse256kMemoryExtension->Enabled = true;
    CheckBoxUse256kSiliconDisc->Checked = false;
    CheckBoxUse256kSiliconDisc->Enabled = true;

    // 64Ko expansion
    switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
    {
      case CPC_MODEL_464_UK:
      case CPC_MODEL_464_FR:
      case CPC_MODEL_464_SP:
      case CPC_MODEL_664_UK:
      case CPC_MODEL_664_FR:
      {
        CheckBoxUse64kMemoryExtension->Checked = false;
        CheckBoxUse64kMemoryExtension->Enabled = true;
      }
      break;

      default:
      {
        CheckBoxUse64kMemoryExtension->Checked = true;
        CheckBoxUse64kMemoryExtension->Enabled = false;
      }
    }
  }

  UpdateSiliconDisc();
#endif /* USE_RAM_4MB */

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::ComboBoxMF2RomDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxMF2Rom->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxMF2Rom->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxMF2Rom->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Index);
  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::PanelSettingsSaveClick(TObject *Sender)
{
  AnsiString NewSettingsFilename = "";

  SaveDialog->Title = "Save settings...";
  SaveDialog->FileName = "";
  SaveDialog->InitialDir = mSettingsP->SettingsDirectory;
  SaveDialog->Filter = SETTINGS_FILTER;
  SaveDialog->DefaultExt = SETTINGS_INI_EXT;
  if (SaveDialog->Execute() == true)
  {
    // Prepare settings file
    NewSettingsFilename = SaveDialog->FileName;
    TSettings *NewSettings = new TSettings(NewSettingsFilename);
    StoreSettings(NewSettings);
    NewSettings->UpdateEmulatorSettings();
    delete NewSettings;
  }

  UpdateAvailableSettings(ExtractFileName(NewSettingsFilename));

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::ComboBoxSettingsChange(
      TObject *Sender)
{
  AnsiString Filename = mSettingsP->SettingsDirectory +  ComboBoxSettings->Text + SETTINGS_INI_EXT;

  // Read settings file
  TSettings *NewSettings = new TSettings(Filename);
  DisplaySettings(NewSettings);
  delete NewSettings;

  ActiveControl = StringGridROMMapping;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::ComboBoxSettingsDrawItem(
      TWinControl *Control, int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxSettings->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxSettings->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxSettings->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Index);
  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::StoreSettings(TSettings* SettingsP)
{
  //
  // Update new settings
  //
  SettingsP->CPCDevice = Devices[ComboBoxDevices->ItemIndex].CPCModel;
  SettingsP->CPCBrand = ComboBoxBrands->ItemIndex;
  SettingsP->CRTCType = ComboBoxCRTCType->ItemIndex;
  SettingsP->UseFloppyDrive = CheckBoxUseFloppyDrive->Checked;
  SettingsP->UseFastFDC = CheckBoxUseFastFDC->Checked;
  SettingsP->Use64kMemoryExtension = CheckBoxUse64kMemoryExtension->Checked;
  SettingsP->Use256kMemoryExtension = CheckBoxUse256kMemoryExtension->Checked;
  SettingsP->Use256kSiliconDisc = CheckBoxUse256kSiliconDisc->Checked;
  SettingsP->Use4MMemoryExtension = CheckBoxUse4MMemoryExtension->Checked;
  SettingsP->UseTechniMusique = CheckBoxUseTechniMusique->Checked;
  SettingsP->UsePlayCity = CheckBoxUsePlayCity->Checked;
  SettingsP->UseDigiblaster = CheckBoxUseDigiblaster->Checked;
  SettingsP->UseMultiface2 = CheckBoxUseMultiface2->Checked;

  //
  // Apply new ROM mapping
  //
  // Lower ROM
  AnsiString NewROM = StringGridROMMapping->Cells[1][0];
#ifndef USE_NEW_ROM_MAPPING_STORAGE
  if (NewROM.Pos(DefaultPrefix) == 1)
  {
    SettingsP->LowerROMFilename = "";
  }
  else
#endif /* USE_NEW_ROM_MAPPING_STORAGE */
  {
    SettingsP->LowerROMFilename = NewROM;
  }
  // Upper ROM
  NewROM = StringGridROMMapping->Cells[1][1];
#ifndef USE_NEW_ROM_MAPPING_STORAGE
  if (NewROM.Pos(DefaultPrefix) == 1)
  {
    SettingsP->UpperROMFilename[0] = "";
  }
  else
#endif /* USE_NEW_ROM_MAPPING_STORAGE */
  {
    SettingsP->UpperROMFilename[0] = NewROM;
  }
#ifndef USE_NEW_ROM_MAPPING_STORAGE
  // AMSDOS
  NewROM = StringGridROMMapping->Cells[1][8];
  if (NewROM == DefaultAMSDOSRomName)
  {
    SettingsP->UpperROMFilename[7] = "";
  }
  else if (NewROM.IsEmpty() == true)
  {
    SettingsP->UpperROMFilename[7] = EmptyRomName;
  }
  else
  {
    SettingsP->UpperROMFilename[7] = NewROM;
  }
#endif /* USE_NEW_ROM_MAPPING_STORAGE*/
  // All others
  for (int loop=1; loop < NB_UPPER_ROMS; loop++)
  {
#ifndef USE_NEW_ROM_MAPPING_STORAGE
    if (loop == 7) continue; // Already treated
#endif /* USE_NEW_ROM_MAPPING_STORAGE */

    NewROM = StringGridROMMapping->Cells[1][loop+1];
    if (NewROM.IsEmpty() == true)
    {
      SettingsP->UpperROMFilename[loop] = EmptyRomName;
    }
    else
    {
      SettingsP->UpperROMFilename[loop] = NewROM;
    }
  }

  //
  // Multiface 2 ROM
  //
  if (ComboBoxMF2Rom->Text == DefaultMultiface2RomName)
  {
    SettingsP->MF2ROMFilename = "";
  }
  else
  {
    SettingsP->MF2ROMFilename = ComboBoxMF2Rom->Text;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormEmulatorSettings::DisplaySettings(TSettings* SettingsP)
{
  //
  // Read settings
  //
  for (int Index=0; Index < NB_DEVICES; Index++)
  {
    if (Devices[Index].CPCModel == SettingsP->CPCDevice)
    {
      ComboBoxDevices->ItemIndex = Index;
      break;
    }
  }
  
  ComboBoxBrands->ItemIndex = SettingsP->CPCBrand;
  ComboBoxCRTCType->ItemIndex = SettingsP->CRTCType;
  CheckBoxUseFloppyDrive->Checked = SettingsP->UseFloppyDrive;
  CheckBoxUse256kMemoryExtension->Checked = SettingsP->Use256kMemoryExtension;
#ifdef USE_SILICON_DISC
  CheckBoxUse256kSiliconDisc->Checked = SettingsP->Use256kSiliconDisc;
#endif /* USE_SILICON_DISC */
#ifdef USE_RAM_4MB
  CheckBoxUse4MMemoryExtension->Checked = SettingsP->Use4MMemoryExtension;
#endif /* USE_RAM_4MB */
  CheckBoxUseTechniMusique->Checked = SettingsP->UseTechniMusique;
  CheckBoxUsePlayCity->Checked = SettingsP->UsePlayCity;
  CheckBoxUseDigiblaster->Checked = SettingsP->UseDigiblaster;
  CheckBoxUseMultiface2->Checked = SettingsP->UseMultiface2;

  // Fast FDC
  if (CheckBoxUseFloppyDrive->Checked == true)
  {
    CheckBoxUseFastFDC->Enabled = true;
    CheckBoxUseFastFDC->Checked = SettingsP->UseFastFDC;
  }
  else
  {
    CheckBoxUseFastFDC->Enabled = false;
    CheckBoxUseFastFDC->Checked = false;
  }

  // 64Ko expansion
  switch (Devices[ComboBoxDevices->ItemIndex].CPCModel)
  {
    case CPC_MODEL_464_UK:
    case CPC_MODEL_464_FR:
    case CPC_MODEL_464_SP:
    case CPC_MODEL_664_UK:
    case CPC_MODEL_664_FR:
    {
      CheckBoxUse64kMemoryExtension->Checked = SettingsP->Use64kMemoryExtension;
      CheckBoxUse64kMemoryExtension->Enabled = true;
    }
    break;

    default:
    {
      CheckBoxUse64kMemoryExtension->Checked = true;
      CheckBoxUse64kMemoryExtension->Enabled = false;
    }
  }

  // Brand name limitations
  ComboBoxBrands->Enabled = (SettingsP->CPCDevice == CPC_MODEL_464_SP) ? false : true;
  if (SettingsP->CPCDevice == CPC_MODEL_464_SP)
  {
    ComboBoxBrands->ItemIndex = 0;
  }

  // CRTC Type
  if ( (SettingsP->CPCDevice == CPC_MODEL_664_UK)
       || (SettingsP->CPCDevice == CPC_MODEL_664_FR) )
  {
    // CRTC0 Only
    ComboBoxCRTCType->ItemIndex = 0;
    ComboBoxCRTCType->Enabled = false;
  }

  //
  // Roms mapping
  //
  FillROMMapping(SettingsP);
  StringGridROMMapping->Row = 0;
  UpdateAvailableROMList(StringGridROMMapping->Row);

  //
  // Multiface 2 Rom
  //
  ComboBoxMF2Rom->ItemIndex = ComboBoxMF2Rom->Items->IndexOf(SettingsP->MF2ROMFilename);
  if (ComboBoxMF2Rom->ItemIndex == -1)
  {
    ComboBoxMF2Rom->ItemIndex = ComboBoxMF2Rom->Items->IndexOf(DefaultMultiface2RomName);
  }

#ifdef USE_RAM_4MB
  if (CheckBoxUse4MMemoryExtension->Checked == true)
  {
    CheckBoxUse64kMemoryExtension->Enabled = false;
    CheckBoxUse256kMemoryExtension->Enabled = false;
#ifdef USE_SILICON_DISC
    CheckBoxUse256kSiliconDisc->Enabled = false;
#endif /* USE_SILICON_DISC */
  }
#endif /* USE_RAM_4MB */
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::PanelButtonMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonTextColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::PanelButtonMouseUp(TObject *Sender, TMouseButton Button,
          TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonTextColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

void __fastcall TFormEmulatorSettings::FormKeyDown(TObject *Sender, WORD &Key, TShiftState Shift)

{
  if (Key == VK_ESCAPE)
  {
    PanelCancelClick(Sender);
  }
  else if (Key == VK_RETURN)
  {
    PanelApplyClick(Sender);
  }
  else
  {
    // Nothing to do
  }

  NOT_USED(Shift);
}
//---------------------------------------------------------------------------

