/*
 CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
 Copyright (C) 2014-2018 by Frdric Coste

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <vcl.h>
#pragma hdrstop
#include "TFormGraphicsExplorer.h"
#include "TFormGraphicsGrid.h"
#include "TFormEditValue.h"
#include "TFormMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"


//=================================
// Options

//=================================


static const AnsiString cTitle = "Graphics explorer";
static const AnsiString cTitlePaused  = " - [PAUSED]";
static const AnsiString cTitleMonitoring  = " - [Monitoring]";
static const AnsiString cTitlePreview  = " - [PREVIEW]";


TFormGraphicsExplorer *FormGraphicsExplorer;
//---------------------------------------------------------------------------
__fastcall TFormGraphicsExplorer::TFormGraphicsExplorer(TComponent* Owner)
  : TForm(Owner)
{
  mEmulatorP = NULL;
  mSettingsP = NULL;

  mZoom = 1;
  mTileX = 2;
  mTileY = 8;
  mMapX = 32;
  mMapY = 8;
  mTopLeftAddress = 0;
  mTileAddress = 0;
  mFirstTileAddress = 0;
  mLastTileAddress = 0;
  mNbTiles = 1;
  mMapStartAddress = 0;

  UpDownZoom->Position = mZoom;
  UpDownTileX->Position = mTileX;
  UpDownTileY->Position = mTileY;
  UpDownMapX->Position = mMapX;
  UpDownMapY->Position = mMapY;
  UpDownMapperTiles->Position = mNbTiles;

  ComboBoxMemory->ItemIndex = 0;
  ComboBoxMode->ItemIndex = 0;
  ComboBoxAddressing->ItemIndex = 0;
  ComboBoxIndexing->ItemIndex = 0;

  // Apply Theme
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("Panel") == 1)
    {
      ((TPanel*)Components[Loop])->Font->Color = clBlack;
    }
    else if (Components[Loop]->Name.Pos("Edit") == 1)
    {
      ((TEdit*)Components[Loop])->Font->Color = clBlack;
      ((TEdit*)Components[Loop])->Font->Size = 12;
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::FormDestroy(TObject *Sender)
{
  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::FormShow(TObject *Sender)
{
  if ( (FormMain == NULL) || (FormGraphicsGrid == NULL) )
  {
    Close();
    return;
  }

  ShowHint = FormMain->ShowHint;

  mMonitoring = false;
  mMapPreview = false;
  UpdateControlAccess();
  mPaused = FormMain->NativeCPC->paused;
  PanelButtonMonitor->Caption = "Start";
  Caption = Title;

  ComboBoxModeChange(Sender);
  ComboBoxAddressingChange(Sender);
  ComboBoxMemoryChange(Sender);
  ComboBoxIndexingChange(Sender);
  UpDownZoom->Position = mZoom;
  UpDownTileX->Position = mTileX;
  UpDownTileY->Position = mTileY;
  PanelFormat->Caption = "&&";
  PanelAddress->Caption = AnsiString::IntToHex(mTopLeftAddress, 6);
  PanelTileAddress->Caption = AnsiString::IntToHex(mTileAddress, 6);
  PanelCursorAddress->Caption = "";
  PanelCursorData->Caption = "";
  PanelFirstTile->Caption = AnsiString::IntToHex(mFirstTileAddress, 6);
  PanelLastTile->Caption = AnsiString::IntToHex(mLastTileAddress, 6);
  PanelMapStart->Caption = AnsiString::IntToHex(mMapStartAddress, 6);
  PageControlExplorer->ActivePage = TabSheetTiles;
  FormMain->SubscribeToCPCEvents(OnStepByStepEvent);

  // Initialise Grid
  FormGraphicsGrid->TileX = mTileX;
  FormGraphicsGrid->TileY = mTileY;
  FormGraphicsGrid->Zoom = mZoom;
  FormGraphicsGrid->MapX = mMapX;
  FormGraphicsGrid->MapY = mMapY;
  FormGraphicsGrid->AddressingMode = 0;
  FormGraphicsGrid->Memory = 0;
  FormGraphicsGrid->Mode = 0;

  // Show Grid
  FormGraphicsGrid->Show();

  if (mSettingsP != NULL)
  {
    for (int Loop=0; Loop < ComponentCount; Loop++)
    {
      if (Components[Loop]->Name.Pos("PanelButton") == 1)
      {
        ((TPanel*)Components[Loop])->Color = mSettingsP->ThemeButtonColor;
        ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeButtonTextColor;
      }
    }
  }

  // Refresh display
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (((TControl*)Components[Loop])->Visible == true)
    {
      ((TControl*)Components[Loop])->Invalidate();
    }
  }

  if (mSettingsP)
  {
    mSettingsP->GraphicsExplorerFormVisible = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::FormHide(TObject *Sender)
{
  // Hide Grid
  FormGraphicsGrid->Hide();

  // Unsubscribe to step by step event
  if (FormMain != NULL)
  {
    FormMain->UnsubscribeToCPCEvents(OnStepByStepEvent);
  }

  if (mSettingsP)
  {
    mSettingsP->GraphicsExplorerFormVisible = false;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


AnsiString __fastcall TFormGraphicsExplorer::GetTitle(void)
{
  AnsiString Title = cTitle;

  if (mMonitoring == true) Title += cTitleMonitoring;
  if (mPaused == true) Title += cTitlePaused;
  if (mMapPreview == true) Title += cTitlePreview;

  return Title;
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetSettings(TSettings* Value)
{
  mSettingsP = Value;

  if (    (mSettingsP->GraphicsExplorerFormLeft != 0)
       || (mSettingsP->GraphicsExplorerFormTop != 0) )
  {
    Position = poDesigned;
    Left = mSettingsP->GraphicsExplorerFormLeft;
    Top = mSettingsP->GraphicsExplorerFormTop;
  }

  // Dispatch settings to Grid
  FormGraphicsGrid->Settings = mSettingsP;

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  Font->Color = mSettingsP->ThemeText1Color;
  LabelGrid->Font->Color = mSettingsP->ThemeText2Color;
  LabelTileAddress->Font->Color = mSettingsP->ThemeText2Color;
  LabelCursorAddress->Font->Color = mSettingsP->ThemeText2Color;
  PanelGrid->Font->Color = mSettingsP->ThemeText1Color;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("PanelButton") == 1)
    {
      ((TPanel*)Components[Loop])->Color = mSettingsP->ThemeButtonColor;
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeButtonTextColor;
    }
    else if (Components[Loop]->Name.Pos("PanelTile") == 1)
    {
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
    else if (Components[Loop]->Name.Pos("PanelCursor") == 1)
    {
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
  }

  if (mSettingsP->HideAll == false)
  {
    Visible = mSettingsP->GraphicsExplorerFormVisible;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::WMMove(TMessage &Message)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->GraphicsExplorerFormLeft = Left;
    mSettingsP->GraphicsExplorerFormTop = Top;
  }

  NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::WMMoving(TMessage &Message)
{
  FormGraphicsGrid->Top = Top + Height;
  FormGraphicsGrid->Left = Left;

  NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::ComboBoxDrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TComboBox* ComboBox = (TComboBox*)Control;
  TCanvas* Canvas = ComboBox->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBox->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBox->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::EditZoomChange(TObject *Sender)
{
  if (EditZoom->Text.IsEmpty() == false)
  {
    mZoom = EditZoom->Text.ToInt();
    if (mZoom < 1) mZoom = 1;

    FormGraphicsGrid->Zoom = mZoom;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsExplorer::EditTileXChange(TObject *Sender)
{
  if (EditTileX->Text.IsEmpty() == false)
  {
    mTileX = EditTileX->Text.ToInt();
    if (mTileX < 1) mTileX = 1;

    FormGraphicsGrid->TileX = mTileX;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsExplorer::EditTileYChange(TObject *Sender)
{
  if (EditTileY->Text.IsEmpty() == false)
  {
    mTileY = EditTileY->Text.ToInt();
    if (mTileY < 1) mTileY = 1;

    FormGraphicsGrid->TileY = mTileY;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::ComboBoxModeChange(TObject *Sender)
{
  FormGraphicsGrid->Mode = ComboBoxMode->ItemIndex;
  FormGraphicsGrid->RefreshGrid();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetTopLeftAddress(tULong Value)
{
  mTopLeftAddress = Value;

  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelAddress->Caption = mTopLeftAddress;
  }
  else
  {
    PanelAddress->Caption = AnsiString::IntToHex(mTopLeftAddress, 6);
  }

  FormGraphicsGrid->UpdateTileAddress();
  FormGraphicsGrid->RefreshGrid();
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetCursorAddress(int Value)
{
  if (Value != -1)
  {
    mCursorAddress = Value;

    // Get Byte Address
    if (PanelFormat->Caption.IsEmpty() == true)
    {
      PanelCursorAddress->Caption = mCursorAddress;
    }
    else
    {
      PanelCursorAddress->Caption = AnsiString::IntToHex(mCursorAddress, 6);
    }
  }
  else
  {
    PanelCursorAddress->Caption = "";
    PanelCursorData->Caption = "";
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::SetCursorData(tUChar Value)
{
  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelCursorData->Caption = Value;
  }
  else
  {
    PanelCursorData->Caption = AnsiString::IntToHex(Value, 2);
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetTileAddress(tULong Value)
{
  mTileAddress = Value;

  bool bHexa;
  if (mMapPreview == false)
  {
    bHexa = (PanelFormat->Caption.IsEmpty() == true) ? false : true;
  }
  else
  {
    bHexa = (PanelFormat->Caption.IsEmpty() == true) ? false : true;
  }

  if (bHexa == false)
  {
    PanelTileAddress->Caption = mTileAddress;
  }
  else
  {
    PanelTileAddress->Caption = AnsiString::IntToHex(mTileAddress, 6);
  }

  // Update Tile Index
  int TileIndex = (int)(mTileAddress - mFirstTileAddress) / (mTileX * mTileY);
  if (mMapPreview == false)
  {
    if (TileIndex >= 0)
    {
      mTileIndex = (tULong)TileIndex;
      PanelTileIndex->Visible = true;

      if (PanelFormat->Caption.IsEmpty() == true)
      {
        PanelTileIndex->Caption = mTileIndex;
      }
      else
      {
        PanelTileIndex->Caption = AnsiString::IntToHex(mTileIndex, 2);
      }
    }
    else
    {
      PanelTileIndex->Visible = false;
    }
  }
  else
  {
    PanelTileIndex->Visible = false;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::SetFirstTileAddress(tULong Value)
{
  mFirstTileAddress = Value;

  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelFirstTile->Caption = mFirstTileAddress;
  }
  else
  {
    PanelFirstTile->Caption = AnsiString::IntToHex(mFirstTileAddress, 6);
  }

  if (mLastTileAddress >= mFirstTileAddress)
  {
    mNbTiles = 1 + ((mLastTileAddress - mFirstTileAddress) / (mTileX * mTileY));
    UpDownMapperTiles->Position = mNbTiles;

    // Update Scrollbar position
    if (mMapPreview == true)
    {
      FormGraphicsGrid->RefreshGrid();
    }
  }
  else
  {
    mNbTiles = 1;
    UpDownMapperTiles->Position = mNbTiles;
  }

  // Update Tile Index
  int TileIndex = (int)(mTileAddress - mFirstTileAddress) / (mTileX * mTileY);
  if (mMapPreview == false)
  {
    if (TileIndex >= 0)
    {
      mTileIndex = (tULong)TileIndex;
      PanelTileIndex->Visible = true;

      if (PanelFormat->Caption.IsEmpty() == true)
      {
        PanelTileIndex->Caption = mTileIndex;
      }
      else
      {
        PanelTileIndex->Caption = AnsiString::IntToHex(mTileIndex, 2);
      }
    }
    else
    {
      PanelTileIndex->Visible = false;
    }
  }
  else
  {
    PanelTileIndex->Visible = false;
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::SetLastTileAddress(tULong Value)
{
  mLastTileAddress = Value;

  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelLastTile->Caption = mLastTileAddress;
  }
  else
  {
    PanelLastTile->Caption = AnsiString::IntToHex(mLastTileAddress, 6);
  }

  if (mLastTileAddress >= mFirstTileAddress)
  {
    mNbTiles = 1 + ((mLastTileAddress - mFirstTileAddress) / (mTileX * mTileY));
    UpDownMapperTiles->Position = mNbTiles;

    // Update Scrollbar position
    if (mMapPreview == true)
    {
      FormGraphicsGrid->RefreshGrid();
    }
  }
  else
  {
    mNbTiles = 1;
    UpDownMapperTiles->Position = mNbTiles;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetMapStartAddress(tULong Value)
{
  mMapStartAddress = Value;

  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelMapStart->Caption = mMapStartAddress;
  }
  else
  {
    PanelMapStart->Caption = AnsiString::IntToHex(mMapStartAddress, 6);
  }

  FormGraphicsGrid->UpdateTileAddress();

  // Update Scrollbar position
  if (mMapPreview == true)
  {
    FormGraphicsGrid->RefreshGrid();
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::SetGridSize(AnsiString Value)
{
  PanelGrid->Caption = Value;
}
//---------------------------------------------------------------------------


bool __fastcall TFormGraphicsExplorer::GetMapXReverse(void)
{
  return PanelMapXReverse->Caption.IsEmpty();
}
//---------------------------------------------------------------------------
bool __fastcall TFormGraphicsExplorer::GetMapYReverse(void)
{
  return PanelMapYReverse->Caption.IsEmpty();
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelFormatClick(TObject *Sender)
{
  if (PanelFormat->Caption.IsEmpty() == true)
  {
    PanelFormat->Caption = "&&";
    PanelAddress->Caption = AnsiString::IntToHex(mTopLeftAddress, 6);
    PanelTileAddress->Caption = AnsiString::IntToHex(mTileAddress, 6);
    PanelTileIndex->Caption = AnsiString::IntToHex(mTileIndex, 2);
    PanelFirstTile->Caption = AnsiString::IntToHex(mFirstTileAddress, 6);
    PanelLastTile->Caption = AnsiString::IntToHex(mLastTileAddress, 6);
    PanelMapStart->Caption = AnsiString::IntToHex(mMapStartAddress, 6);
  }
  else
  {
    PanelFormat->Caption = "";
    PanelAddress->Caption = mTopLeftAddress;
    PanelTileAddress->Caption = mTileAddress;
    PanelTileIndex->Caption = mTileIndex;
    PanelFirstTile->Caption = mFirstTileAddress;
    PanelLastTile->Caption = mLastTileAddress;
    PanelMapStart->Caption = mMapStartAddress;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelAddressClick(TObject *Sender)
{
  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mTopLeftAddress;
  EditValue->Hexadecimal = true;
  EditValue->NbDigits = 6;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mRAMMax;
  if (EditValue->ShowModal() == mrOk)
  {
    TopLeftAddress = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonSetTopLeftClick(TObject *Sender)
{
  TopLeftAddress = TileAddress;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonRefreshClick(TObject *Sender)
{
  ComboBoxModeChange(Sender);

  FormGraphicsGrid->RefreshGrid();
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonEditTileClick(TObject *Sender)
{
  FormGraphicsGrid->EditTile();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelButtonMapPreviewClick(TObject *Sender)
{
  // Invert Map Preview
  mMapPreview = mMapPreview == true ? false : true;

  Caption = Title;
  UpdateControlAccess();

  FormGraphicsGrid->MapPreview = mMapPreview;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::ComboBoxAddressingChange(TObject *Sender)
{
  FormGraphicsGrid->AddressingMode = ComboBoxAddressing->ItemIndex;
  FormGraphicsGrid->RefreshGrid();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::ComboBoxMemoryChange(TObject *Sender)
{
  switch (ComboBoxMemory->ItemIndex)
  {
    case 0:
    case 1:
    {
      mRAMMax = 65535;
    }
    break;

    default:
    {
      mRAMMax = (FormMain->NativeCPC->ram_size * 1024) - 1;
    }
    break;
  }

  FormGraphicsGrid->Memory = ComboBoxMemory->ItemIndex;
  FormGraphicsGrid->RAMMax = mRAMMax;
  FormGraphicsGrid->RefreshGrid();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonMonitorClick(TObject *Sender)
{
  mMonitoring = mMonitoring == true ? false : true;
  Caption = Title;

  if (mMonitoring == true)
  {
    PanelButtonMonitor->Caption = "Stop";
  }
  else
  {
    PanelButtonMonitor->Caption = "Start";
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


bool __fastcall TFormGraphicsExplorer::OnStepByStepEvent(tULong Condition, tNativeCPC* NativeCPC)
{
  bool Pause = NativeCPC->paused;

  if (mPaused != Pause)
  {
    mPaused = Pause;
    Caption = Title;
  }

  if ( (mMonitoring == true)
       && (Condition & (EC_FRAME_COMPLETE | EC_END_INSTRUCTION)) )
  {
    FormGraphicsGrid->RefreshGrid();
  }

  if (Condition & EC_RESET)
  {
    // Refresh RAM content display
    ComboBoxMemoryChange(NULL);
  }


  return Pause;
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PageControlExplorerDrawTab(TCustomTabControl *Control,
          int TabIndex, const TRect &Rect, bool Active)
{
  TCanvas* Canvas = PageControlExplorer->Canvas;

  if (Active == true)
  {
    Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }

  // Get sheet corresponding to TabIndex
  int TabCount=0;
  TTabSheet* Sheet = NULL;
  for (int Loop=0; Loop < PageControlExplorer->PageCount; Loop++)
  {
    if (PageControlExplorer->Pages[Loop]->TabVisible == true)
    {
      if (TabCount == TabIndex)
      {
        Sheet = PageControlExplorer->Pages[Loop];
        break;
      }

      TabCount++;
    }
  }

  // Draw tab
  if (Sheet != NULL)
  {
    Canvas->FillRect(Rect);
    AnsiString Text = Sheet->Caption;
    int TextWidth = Canvas->TextWidth(Text);
    int TextHeight = Canvas->TextHeight(Text);
    int TextPosX = (Rect.Width() - TextWidth) / 2;
    int TextPosY = (PageControlExplorer->TabHeight - TextHeight) / 2;
    Canvas->TextOut(Rect.Left+TextPosX, Rect.Top+TextPosY, Text);
  }

  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelFirstTileClick(TObject *Sender)
{
  if (mMapPreview == true) return;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mFirstTileAddress;
  EditValue->Hexadecimal = true;
  EditValue->NbDigits = 6;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mRAMMax;
  EditValue->Increment = mTileX * mTileY;
  if (EditValue->ShowModal() == mrOk)
  {
    FirstTileAddress = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelButtonSetFirstTileClick(TObject *Sender)
{
  FirstTileAddress = TileAddress;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelLastTileClick(TObject *Sender)
{
  if (mMapPreview == true) return;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mLastTileAddress;
  EditValue->Hexadecimal = true;
  EditValue->NbDigits = 6;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mRAMMax;
  EditValue->Increment = mTileX * mTileY;
  if (EditValue->ShowModal() == mrOk)
  {
    LastTileAddress = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonSetLastTileClick(TObject *Sender)
{
  LastTileAddress = TileAddress;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::EditMapperTilesChange(TObject *Sender)
{
  if (EditMapperTiles->Text.IsEmpty() == false)
  {
    mNbTiles = EditMapperTiles->Text.ToInt();
    if (mNbTiles < 1) mNbTiles = 1;

    // Update Last Tile Address
    LastTileAddress = mFirstTileAddress + ((mNbTiles - 1) * (mTileX * mTileY));
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------



void __fastcall TFormGraphicsExplorer::PanelMapStartClick(TObject *Sender)
{
  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = mMapStartAddress;
  EditValue->Hexadecimal = true;
  EditValue->NbDigits = 6;
  EditValue->MinValue = 0;
  EditValue->MaxValue = mRAMMax;
  if (EditValue->ShowModal() == mrOk)
  {
    MapStartAddress = EditValue->NewValue;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelButtonSetMapStartClick(TObject *Sender)
{
  MapStartAddress = TileAddress;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::EditMapXChange(TObject *Sender)
{
  if (EditMapX->Text.IsEmpty() == false)
  {
    mMapX = EditMapX->Text.ToInt();
    if (mMapX < 1) mMapX = 1;

    if (mMapPreview == true)
    {
      FormGraphicsGrid->RefreshGrid();
    }

    FormGraphicsGrid->MapX = mMapX;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelMapXReverseClick(TObject *Sender)
{
  if (PanelMapXReverse->Caption.IsEmpty() == true)
  {
    PanelMapXReverse->Caption = "R";
  }
  else
  {
    PanelMapXReverse->Caption = "";
  }

  if (mMapPreview == true)
  {
    FormGraphicsGrid->UpdateTileAddress();
    FormGraphicsGrid->RefreshGrid();
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::EditMapYChange(TObject *Sender)
{
  if (EditMapY->Text.IsEmpty() == false)
  {
    mMapY = EditMapY->Text.ToInt();
    if (mMapY < 1) mMapY = 1;

    if (mMapPreview == true)
    {
      FormGraphicsGrid->RefreshGrid();
    }

    FormGraphicsGrid->MapY = mMapY;
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelMapYReverseClick(TObject *Sender)
{
  if (PanelMapYReverse->Caption.IsEmpty() == true)
  {
    PanelMapYReverse->Caption = "R";
  }
  else
  {
    PanelMapYReverse->Caption = "";
  }

  if (mMapPreview == true)
  {
    FormGraphicsGrid->UpdateTileAddress();
    FormGraphicsGrid->RefreshGrid();
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PageControlExplorerChanging(TObject *Sender,
          bool &AllowChange)
{
  if (mMapPreview == true)
  {
    AllowChange = false;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonMapperCopyClick(TObject *Sender)
{
  FormGraphicsGrid->MapperCopy();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::ComboBoxIndexingChange(TObject *Sender)
{
  FormGraphicsGrid->MapperIndexing = ComboBoxIndexing->ItemIndex;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::PanelButtonMouseDown(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonTextColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsExplorer::PanelButtonMouseUp(TObject *Sender,
          TMouseButton Button, TShiftState Shift, int X, int Y)
{
  ((TPanel*)Sender)->Color = mSettingsP->ThemeButtonColor;
  ((TPanel*)Sender)->Font->Color = mSettingsP->ThemeButtonTextColor;

  NOT_USED(Button);
  NOT_USED(Shift);
  NOT_USED(X);
  NOT_USED(Y);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsExplorer::UpdateControlAccess(void)
{
  // Update controls accesses
  bool ShowWhenEdit = (mMapPreview == true) ? false : true;
  LabelGrid->Visible = ShowWhenEdit;
  PanelGrid->Visible = ShowWhenEdit;
  LabelTopLeftAddress->Visible = ShowWhenEdit;
  PanelFormat->Visible = ShowWhenEdit;
  PanelAddress->Visible = ShowWhenEdit;
  LabelCursorAddress->Visible = ShowWhenEdit;
  PanelCursorAddress->Visible = ShowWhenEdit;
  PanelCursorData->Visible = ShowWhenEdit;
  PanelButtonSetTopLeft->Visible = ShowWhenEdit;
  PanelButtonSetFirstTile->Visible = ShowWhenEdit;
  PanelButtonSetLastTile->Visible = ShowWhenEdit;

  if (mMapPreview == false)
  {
    LabelTileAddress->Visible = true;
    PanelTileAddress->Visible = true;
    PanelButtonMapPreview->Color = mSettingsP->ThemeButtonColor;
    PanelButtonMapPreview->Font->Color = mSettingsP->ThemeButtonTextColor;
    EditMapperTiles->Enabled = true;
  }
  else
  {
    PanelButtonMapPreview->Color = mSettingsP->ThemeButtonTextColor;
    PanelButtonMapPreview->Font->Color = mSettingsP->ThemeButtonColor;
    EditMapperTiles->Enabled = false;
  }
}
//---------------------------------------------------------------------------

