/*
 CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
 Copyright (C) 2014-2018 by Frdric Coste

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <vcl.h>
#include <Clipbrd.hpp>
#include <tchar.h>
#pragma hdrstop
#include "TFormGraphicsGrid.h"
#include "TFormGraphicsExplorer.h"
#include "TFormMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"




TFormGraphicsGrid *FormGraphicsGrid;
//---------------------------------------------------------------------------
__fastcall TFormGraphicsGrid::TFormGraphicsGrid(TComponent* Owner)
  : TForm(Owner)
{
  // Remove Caption
  LONG Style = GetWindowLong(Handle, GWL_STYLE);
  Style = Style & ~(WS_CAPTION);
  SetWindowLong(Handle, GWL_STYLE, Style);

  mZoom = 1;
  mTileX = 2;
  mTileY = 8;
  mMapX = 32;
  mMapY = 8;
  mDrawCells = DrawStringGridTiles;
  mReadData = NULL;
  mWriteData = NULL;
  mMapPreviewTiles = NULL;
  mMapPreview = false;
  mMapPreviewTiles = new TGraphicsGridTilePreview[mMapX * mMapY];
  mDrawMapPreview = UpdateMapPreviewDataLinearBytes;

  // Apply Theme
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::FormDestroy(TObject *Sender)
{
  delete [] mMapPreviewTiles;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::FormShow(TObject *Sender)
{
  if (FormGraphicsExplorer == NULL)
  {
    Close();
    return;
  }

  Top = FormGraphicsExplorer->Top + FormGraphicsExplorer->Height;
  Left = FormGraphicsExplorer->Left;

  mMapPreview = false;
  mDrawCells = DrawStringGridTiles;
  GraphicsEditorP = new TFormGraphicsEditor(this);

  ResizeTiles();
  UpdatePalette();
  UpdateMapPreview();

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::FormHide(TObject *Sender)

{
  delete GraphicsEditorP;
  GraphicsEditorP = NULL;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::FormResize(TObject *Sender)
{
  // Maintain window position
  Top = FormGraphicsExplorer->Top + FormGraphicsExplorer->Height;
  Left = FormGraphicsExplorer->Left;

  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->GraphicsExplorerFormWidth = Width;
    mSettingsP->GraphicsExplorerFormHeight = Height;

    ResizeTiles();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::WMMove(TMessage &Message)
{
	if (Visible == true)
	{
    // Maintain window position
    Top = FormGraphicsExplorer->Top + FormGraphicsExplorer->Height;
    Left = FormGraphicsExplorer->Left;
	}

	NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::WMSysCommand(TMessage &Message)
{
  if ( (Message.Msg == WM_SYSCOMMAND) && (Message.WParam == SC_MOVE) )
  {
    // Nothing to do
  }
  else
  {
    TForm::WndProc(Message);
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::RefreshGrid(void)
{
  UpdatePalette();
  UpdateMapPreview();
  StringGridTiles->Refresh();
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::SetSettings(TSettings* Value)
{
  mSettingsP = Value;

  if (    (mSettingsP->GraphicsExplorerFormWidth != 0)
       || (mSettingsP->GraphicsExplorerFormHeight != 0) )
  {
    Width = mSettingsP->GraphicsExplorerFormWidth;
    Height = mSettingsP->GraphicsExplorerFormHeight;
  }

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  Font->Color = mSettingsP->ThemeText1Color;
  StringGridTiles->Color = mSettingsP->ThemeBackgroundColor;
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::SetAddressingMode(int Value)
{
  mAddressingMode = Value;

  switch (mAddressingMode)
  {
    case 0:
    {
      mGetByteAddress = GetByteAddressLinear;
      mGetTileAddress = GetTileAddressLinear;
    }
    break;

    case 1:
    {
      mGetByteAddress = GetByteAddressScreen;
      mGetTileAddress = GetTileAddressScreen;
      FormGraphicsExplorer->TopLeftAddress = 0xC000;
    }
    break;
  }

  // Update Scrollbar position
  if (mMapPreview == false)
  {
    ScrollBarAddress->Position = FormGraphicsExplorer->TopLeftAddress;
  }
  else
  {
    ScrollBarAddress->Position = FormGraphicsExplorer->MapStartAddress;
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMemory(int Value)
{
  switch (Value)
  {
    case 0:
    {
      mReadData = Engine_ReadBankedRead;
      mWriteData = Engine_WriteBankedRead;
    }
    break;

    case 1:
    {
      mReadData = Engine_ReadBankedWrite;
      mWriteData = Engine_WriteBankedWrite;
    }
    break;

    default:
    {
      mReadData = Engine_ReadRAM;
      mWriteData = Engine_WriteRAM;
    }
    break;
  }

  UpdateMapPreview();
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMapperIndexing(int Value)
{
  switch (Value)
  {
    // Compresses RL0
    case 1:
    {
      mDrawMapPreview = UpdateMapPreviewDataCompressedRL0;
    }
    break;

    // Linear words Big Endian
    case 2:
    {
      mDrawMapPreview = UpdateMapPreviewDataLinearWordsBigEndian;
    }
    break;

    // Compressed & Linear
    case 3:
    {
      mDrawMapPreview = UpdateMapPreviewDataCompressedLinear;
    }
    break;

    // Linear bytes
    default:
    {
      mDrawMapPreview = UpdateMapPreviewDataLinearBytes;
    }
    break;
  }

  UpdateMapPreview();

  if (mMapPreview == true)
  {
    StringGridTiles->Refresh();
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMapPreview(bool Value)
{
  mMapPreview=Value;

  StringGridTiles->ColCount=1;
  StringGridTiles->RowCount=1;
  StringGridTiles->Col = 0;
  StringGridTiles->Row = 0;
  StringGridTiles->ScrollBars = (mMapPreview == true) ? ssBoth : ssNone;

  if (mMapPreview == true)
  {
    mDrawCells = DrawStringGridMapPreview;
    ScrollBarAddress->Position = FormGraphicsExplorer->MapStartAddress;
  }
  else
  {
    mDrawCells = DrawStringGridTiles;
    ScrollBarAddress->Position = FormGraphicsExplorer->TopLeftAddress;
  }

  UpdateMapPreview();
  UpdateTileAddress();
  ResizeTiles();
}
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetTileX(tUChar Value)
{
  mTileX = Value;
  ResizeTiles();
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetTileY(tUChar Value)
{
  mTileY = Value;
  ResizeTiles();
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetZoom(tUChar Value)
{
  mZoom = Value;
  ResizeTiles();
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMapX(tULong Value)
{
  mMapX = Value;

  delete [] mMapPreviewTiles;
  mMapPreviewTiles = new TGraphicsGridTilePreview[mMapX * mMapY];

  UpdateMapPreview();
  ResizeTiles();
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMapY(tULong Value)
{
  mMapY = Value;

  delete [] mMapPreviewTiles;
  mMapPreviewTiles = new TGraphicsGridTilePreview[mMapX * mMapY];

  UpdateMapPreview();
  ResizeTiles();
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetMode(int Value)
{
  if (Value == 0)
  {
    // Read current mode
    mMode = (tUChar)FormMain->NativeCPC->GateArray->scr_mode;
  }
  else
  {
    mMode = (tUChar)(Value - 1);
  }

  switch (mMode)
  {
    case 0:
    {
      mDrawByte = DrawByteMode0;
      mGetPixelColor = GetPixelColorMode0;
    }
    break;

    case 1:
    {
      mDrawByte = DrawByteMode1;
      mGetPixelColor = GetPixelColorMode1;
    }
    break;

    case 2:
    {
      mDrawByte = DrawByteMode2;
      mGetPixelColor = GetPixelColorMode2;
    }
    break;
  }
};
//---------------------------------------------------------------------------
void __fastcall TFormGraphicsGrid::SetRAMMax(tULong Value)
{
  mRAMMax = Value;

  if (ScrollBarAddress->Position > (int)mRAMMax) ScrollBarAddress->Position = (int)mRAMMax;
  ScrollBarAddress->Max = mRAMMax;

  UpdateMapPreview();
};
//---------------------------------------------------------------------------


tULong __fastcall TFormGraphicsGrid::GetTileAddressLinear(tULong Address, int X, int Y)
{
  return (Address
          + (X * mTileY * mTileX * StringGridTiles->RowCount)
          + (Y * mTileY * mTileX));
}
//---------------------------------------------------------------------------
tULong __fastcall TFormGraphicsGrid::GetByteAddressLinear(tULong Address, int X, int Y)
{
  return (Address + (Y * mTileX) + X);
}
//---------------------------------------------------------------------------

tULong __fastcall TFormGraphicsGrid::GetTileAddressScreen(tULong Address, int X, int Y)
{
  return (Address
          + (X * mTileX)
          + (Y * mTileX * StringGridTiles->ColCount));
}
//---------------------------------------------------------------------------
tULong __fastcall TFormGraphicsGrid::GetByteAddressScreen(tULong Address, int X, int Y)
{
  return (Address + (0x800 * Y) + X);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::ResizeTiles(void)
{
  int SizeX = (8 * mZoom * mTileX) + 2; // Add 1 pixel margin
  int SizeY = (2 * mZoom * mTileY) + 2; // Add 1 pixel margin

  // Update tile size
  StringGridTiles->DefaultColWidth = SizeX;
  StringGridTiles->DefaultRowHeight = SizeY;

  // Update number of tiles shown
  if (mMapPreview == false)
  {
    StringGridTiles->ColCount = StringGridTiles->Width / SizeX;
    StringGridTiles->RowCount = StringGridTiles->Height / SizeY;
  }
  else
  {
    StringGridTiles->ColCount = mMapX;
    StringGridTiles->RowCount = mMapY;
  }

  if (mMapPreview == false)
  {
    // Linear addressing ?
    if (mAddressingMode == 0)
    {
      ScrollBarAddress->PageSize =  StringGridTiles->ColCount * StringGridTiles->RowCount *
                                    mTileX * mTileY;
      ScrollBarAddress->SmallChange = StringGridTiles->RowCount * mTileX * mTileY;
    }
    else
    {
      ScrollBarAddress->PageSize =  StringGridTiles->ColCount * StringGridTiles->RowCount *
                                    mTileX;
      ScrollBarAddress->SmallChange = StringGridTiles->ColCount * mTileX;
    }

    // Update grid size
    AnsiString GridSize = AnsiString(StringGridTiles->ColCount)
                          + "x"
                          + AnsiString(StringGridTiles->RowCount);
    FormGraphicsExplorer->GridSize = GridSize;
  }

  UpdateTileAddress();
  StringGridTiles->Refresh();

  // Prevent scrollbar update display bug
  if (ScrollBarAddress->Focused() == true)
  {
    StringGridTiles->SetFocus();
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::StringGridTilesDrawCell(TObject *Sender, int ACol,
          int ARow, TRect &Rect, TGridDrawState State)
{
  mDrawCells(ACol, ARow, Rect, State);
  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::UpdatePalette(void)
{
  tNativeCPC* NativeCPC = FormMain->NativeCPC;

  for (int Loop=0; Loop < GA_INK_VALUES_NBELEMENT; Loop++)
  {
    colours_rgb_entry Color = NativeCPC->colours_rgb[NativeCPC->GateArray->ink_values[Loop]];
    mColor[Loop] = (TColor)RGB(Color.colour_r, Color.colour_g, Color.colour_b);

    // Negative grayscale
    int GrayScale = 255 - ((Color.colour_r + Color.colour_g + Color.colour_b) / 3);
    mGrayscaleColor[Loop] = (TColor)RGB(GrayScale, GrayScale, GrayScale);
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::DrawStringGridTiles(int ACol,
          int ARow, TRect &Rect, TGridDrawState State)
{
  TCanvas* Canvas = StringGridTiles->Canvas;

  int BaseAddress = mGetTileAddress(FormGraphicsExplorer->TopLeftAddress, ACol, ARow);
  if (BaseAddress <= (int)mRAMMax)
  {
    // Draw frame
    if (State.Contains(gdSelected))
    {
      Canvas->Brush->Color = mSettingsP->ThemeText1Color;
      Canvas->FrameRect(Rect);
      mColorsP = mGrayscaleColor;
    }
    else
    {
      Canvas->Pen->Color = clLtGray;
      Canvas->MoveTo(Rect.left, Rect.top);
      Canvas->LineTo(Rect.right-1, Rect.top);
      Canvas->LineTo(Rect.right-1, Rect.bottom-1);
      Canvas->Pen->Color = clDkGray;
      Canvas->LineTo(Rect.left, Rect.bottom-1);
      Canvas->LineTo(Rect.left, Rect.top);
      mColorsP = mColor;
    }

    int Y = Rect.Top + 1;
    for (int LoopY = 0; LoopY < mTileY; LoopY++)
    {
      int X = Rect.Left + 1;
      for (int LoopX = 0; LoopX < mTileX; LoopX++)
      {
        tULong Address = mGetByteAddress(BaseAddress, LoopX, LoopY);
        if (Address <= mRAMMax)
        {
          // Read Byte
          tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, Address);
          // Draw Byte
          X = mDrawByte(Data, X, Y);
        }
        else
        {
          // Empty Byte
          Canvas->Brush->Color = mSettingsP->ThemeBackgroundColor;
          Canvas->FillRect(TRect(X, Y, X+(mZoom*8), Y+mZoom*2));
          X += 8*mZoom;
        }
      }

      Y += 2*mZoom;
    }
  }
  else
  {
    // Empty tile
    Canvas->Brush->Color = mSettingsP->ThemeBackgroundColor;
    Canvas->FillRect(Rect);
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::DrawStringGridMapPreview(int ACol,
          int ARow, TRect &Rect, TGridDrawState State)
{
  TCanvas* Canvas = StringGridTiles->Canvas;

  int Index = ACol + (ARow * StringGridTiles->ColCount);
  int Address = FormGraphicsExplorer->MapStartAddress + Index;
  if (Address <= (int)mRAMMax)
  {
    tUShort Data = mMapPreviewTiles[Index].Index;
    if (Data < (int)FormGraphicsExplorer->NbTiles)
    {
      tULong TileAddress = FormGraphicsExplorer->FirstTileAddress + (Data * mTileX * mTileY);

      // Draw frame
      if (State.Contains(gdSelected))
      {
        Canvas->Brush->Color = mSettingsP->ThemeText1Color;
        Canvas->FrameRect(Rect);
      }
      else
      {
        Canvas->Pen->Color = clLtGray;
        Canvas->MoveTo(Rect.left, Rect.top);
        Canvas->LineTo(Rect.right-1, Rect.top);
        Canvas->LineTo(Rect.right-1, Rect.bottom-1);
        Canvas->Pen->Color = clDkGray;
        Canvas->LineTo(Rect.left, Rect.bottom-1);
        Canvas->LineTo(Rect.left, Rect.top);
      }

      mColorsP = mColor;

      // Draw Tile
      int Y = Rect.Top + 1;
      for (int LoopY = 0; LoopY < mTileY; LoopY++)
      {
        int X = Rect.Left + 1;
        for (int LoopX = 0; LoopX < mTileX; LoopX++)
        {
          tULong Address = mGetByteAddress(TileAddress, LoopX, LoopY);
          if (Address <= mRAMMax)
          {
            // Read Byte
            tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, Address);
            // Draw Byte
            X = mDrawByte(Data, X, Y);
          }
          else
          {
            // Empty Byte
            Canvas->Brush->Color = mSettingsP->ThemeBackgroundColor;
            Canvas->FillRect(TRect(X, Y, X+(mZoom*8), Y+mZoom*2));
            X += 8*mZoom;
          }
        }

        Y += 2*mZoom;
      }

    }
    else
    {
      // Empty tile
      Canvas->Brush->Color = mSettingsP->ThemeBackgroundColor;
      Canvas->FillRect(Rect);

      if (State.Contains(gdSelected))
      {
        Canvas->Brush->Color = mSettingsP->ThemeText1Color;
        Canvas->FrameRect(Rect);
        mColorsP = mGrayscaleColor;
      }
    }
  }
  else
  {
    // Empty tile
    Canvas->Brush->Color = mSettingsP->ThemeBackgroundColor;
    Canvas->FillRect(Rect);

    if (State.Contains(gdSelected))
    {
      Canvas->Brush->Color = mSettingsP->ThemeText1Color;
      Canvas->FrameRect(Rect);
      mColorsP = mGrayscaleColor;
    }
  }

  NOT_USED(State);
}
//---------------------------------------------------------------------------


TColor __fastcall TFormGraphicsGrid::GetPixelColorMode0(tUChar Data, int X)
{
  tUChar Color;
  switch (X % 2)
  {
    case 0:
    {
      Color =   ((Data & 0x02) << 2) | ((Data & 0x20) >> 3)
              | ((Data & 0x08) >> 2) | ((Data & 0x80) >> 7);
    }
    break;
    case 1:
    {
      Color =   ((Data & 0x01) << 3) | ((Data & 0x10) >> 2)
              | ((Data & 0x04) >> 1) | ((Data & 0x40) >> 6);
    }
    break;
  }

  return (mColor[Color]);
}
//---------------------------------------------------------------------------
TColor __fastcall TFormGraphicsGrid::GetPixelColorMode1(tUChar Data, int X)
{
  tUChar Color;
  switch (X % 4)
  {
    case 0: Color = ((Data & 0x08) >> 2) + ((Data & 0x80) >> 7); break;
    case 1: Color = ((Data & 0x04) >> 1) + ((Data & 0x40) >> 6); break;
    case 2: Color = ((Data & 0x02) >> 0) + ((Data & 0x20) >> 5); break;
    case 3: Color = ((Data & 0x01) << 1) + ((Data & 0x10) >> 4); break;
  }

  return (mColor[Color]);
}
//---------------------------------------------------------------------------
TColor __fastcall TFormGraphicsGrid::GetPixelColorMode2(tUChar Data, int X)
{
  return (mColor[(Data >> (7 - (X % 8))) & 0x01]);
}
//---------------------------------------------------------------------------


int __fastcall TFormGraphicsGrid::DrawByteMode0(tUChar Data, int X, int Y)
{
  tUChar Pixel0 =   ((Data & 0x02) << 2)
                  | ((Data & 0x20) >> 3)
                  | ((Data & 0x08) >> 2)
                  | ((Data & 0x80) >> 7);
  tUChar Pixel1 =   ((Data & 0x01) << 3)
                  | ((Data & 0x10) >> 2)
                  | ((Data & 0x04) >> 1)
                  | ((Data & 0x40) >> 6);

  // Draw 2 pixels
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel0];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*4), Y+mZoom*2));
  X += 4*mZoom;
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel1];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*4), Y+mZoom*2));
  X += 4*mZoom;

  return X;
}
//---------------------------------------------------------------------------
int __fastcall TFormGraphicsGrid::DrawByteMode1(tUChar Data, int X, int Y)
{
  tUChar Pixel1 = ((Data & 0x08) >> 2) + ((Data & 0x80) >> 7);
  tUChar Pixel2 = ((Data & 0x04) >> 1) + ((Data & 0x40) >> 6);
  tUChar Pixel3 = ((Data & 0x02) >> 0) + ((Data & 0x20) >> 5);
  tUChar Pixel4 = ((Data & 0x01) << 1) + ((Data & 0x10) >> 4);

  // Draw 4 pixels
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel1];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*2), Y+mZoom*2));
  X += 2*mZoom;
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel2];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*2), Y+mZoom*2));
  X += 2*mZoom;
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel3];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*2), Y+mZoom*2));
  X += 2*mZoom;
  StringGridTiles->Canvas->Brush->Color = mColorsP[Pixel4];
  StringGridTiles->Canvas->FillRect(TRect(X, Y, X+(mZoom*2), Y+mZoom*2));
  X += 2*mZoom;

  return X;
}
//---------------------------------------------------------------------------
int __fastcall TFormGraphicsGrid::DrawByteMode2(tUChar Data, int X, int Y)
{
  // Draw 8 pixels
  for (int Loop=0; Loop < 8; Loop++)
  {
    StringGridTiles->Canvas->Brush->Color = mColorsP[Data & 0x80 ? 1 : 0];
    StringGridTiles->Canvas->FillRect(TRect(X, Y, X+mZoom, Y+mZoom*2));
    X += mZoom;
    Data = Data << 1;
  }

  return X;
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::ScrollBarAddressChange(TObject *Sender)
{
  if (ScrollBarAddress->Focused() == true)
  {
    if (mMapPreview == false)
    {
      FormGraphicsExplorer->TopLeftAddress = ScrollBarAddress->Position;
    }
    else
    {
      FormGraphicsExplorer->MapStartAddress = ScrollBarAddress->Position;
    }
    StringGridTiles->Refresh();
    UpdateTileAddress();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::ButtonByteDecrementClick(TObject *Sender)
{
  if (mMapPreview == false)
  {
    if (FormGraphicsExplorer->TopLeftAddress > 0)
    {
      FormGraphicsExplorer->TopLeftAddress--;
    }
  }
  else
  {
    if (FormGraphicsExplorer->MapStartAddress > 0)
    {
      FormGraphicsExplorer->MapStartAddress--;
    }
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::ButtonByteIncrementClick(TObject *Sender)
{
  if (mMapPreview == false)
  {
    if (FormGraphicsExplorer->TopLeftAddress < mRAMMax)
    {
      FormGraphicsExplorer->TopLeftAddress++;
    }
  }
  else
  {
    if (FormGraphicsExplorer->MapStartAddress < mRAMMax)
    {
      FormGraphicsExplorer->MapStartAddress++;
    }
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::StringGridTilesSelectCell(TObject *Sender, int ACol,
          int ARow, bool &CanSelect)
{
  if (mMapPreview == false)
  {
    FormGraphicsExplorer->TileAddress = mGetTileAddress(FormGraphicsExplorer->TopLeftAddress, ACol, ARow);
  }
  else
  {
    int Index = ACol + (ARow * mMapX);
    FormGraphicsExplorer->TileAddress = mMapPreviewTiles[Index].Address;
  }

  NOT_USED(Sender);
  NOT_USED(CanSelect);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::StringGridTilesDblClick(TObject *Sender)
{
  if (mMapPreview == false)
  {
    EditTile();
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::StringGridTilesMouseLeave(TObject *Sender)
{
  FormGraphicsExplorer->CursorAddress = -1;

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::StringGridTilesMouseMove(TObject *Sender, TShiftState Shift,
          int X, int Y)
{
  int ACol = X / StringGridTiles->DefaultColWidth;
  int ARow = Y / StringGridTiles->DefaultRowHeight;

  if ( (ACol < StringGridTiles->ColCount) && (ARow < StringGridTiles->RowCount) )
  {
    // Update cursor address
    int PixelX = X - (ACol * StringGridTiles->DefaultColWidth);
    PixelX = PixelX / ((StringGridTiles->DefaultColWidth-2) / mTileX);
    if (PixelX > (mTileX-1))
    {
      PixelX = mTileX-1;
    }
    int PixelY = Y - (ARow * StringGridTiles->DefaultRowHeight);
    PixelY = PixelY / ((StringGridTiles->DefaultRowHeight-2) / mTileY);
    if (PixelY > (mTileY-1))
    {
      PixelY = mTileY-1;
    }

    FormGraphicsExplorer->CursorAddress = mGetByteAddress(
          mGetTileAddress(FormGraphicsExplorer->TopLeftAddress, ACol, ARow), PixelX, PixelY);

    tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, FormGraphicsExplorer->CursorAddress);
    FormGraphicsExplorer->CursorData = Data;
  }
  else
  {
    FormGraphicsExplorer->CursorAddress = -1;
  }

  NOT_USED(Sender);
  NOT_USED(Shift);
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::UpdateMapPreview(void)
{
  if (FormMain->NativeCPC == NULL) return;
  if (mReadData == NULL) return;

  delete [] mMapPreviewTiles;
  mMapPreviewTiles = new TGraphicsGridTilePreview[mMapX * mMapY];

  Screen->Cursor = crHourGlass;

  mPreviewAddress = FormGraphicsExplorer->MapStartAddress;
  mPreviewOccurences = 0;
  mPreviewIncrements = 0;

  // Forward X
  if (FormGraphicsExplorer->MapXReverse == true)
  {
    // Forward Y
    if (FormGraphicsExplorer->MapYReverse == true)
    {
      for (int LoopY=0; LoopY < (int)mMapY; LoopY++)
      {
        for (int LoopX=0; LoopX < (int)mMapX; LoopX++)
        {
          if (mPreviewAddress <= mRAMMax)
          {
            mDrawMapPreview((LoopY * mMapX) + LoopX);
          }
        }
      }
    }
    // Reverse Y
    else
    {
      for (int LoopY=(int)mMapY; LoopY; LoopY--)
      {
        for (int LoopX=0; LoopX < (int)mMapX; LoopX++)
        {
          if (mPreviewAddress <= mRAMMax)
          {
            mDrawMapPreview(((LoopY-1) * mMapX) + LoopX);
          }
        }
      }
    }
  }
  // Reverse X
  else
  {
    // Forward Y
    if (FormGraphicsExplorer->MapYReverse == true)
    {
      for (int LoopY=0; LoopY < (int)mMapY; LoopY++)
      {
        for (int LoopX=(int)mMapX; LoopX; LoopX--)
        {
          if (mPreviewAddress <= mRAMMax)
          {
            mDrawMapPreview((LoopY * mMapX) + (LoopX-1));
          }
        }
      }
    }
    // Reverse Y
    else
    {
      for (int LoopY=(int)mMapY; LoopY; LoopY--)
      {
        for (int LoopX=(int)mMapX; LoopX; LoopX--)
        {
          if (mPreviewAddress <= mRAMMax)
          {
            mDrawMapPreview(((LoopY-1) * mMapX) + (LoopX-1));
          }
        }
      }
    }
  }

  Screen->Cursor = crDefault;
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::UpdateMapPreviewDataLinearBytes(int Index)
{
  mMapPreviewTiles[Index].Index = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress) % FormGraphicsExplorer->NbTiles;
  mMapPreviewTiles[Index].Address = mPreviewAddress;
  mPreviewAddress++;
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::UpdateMapPreviewDataCompressedRL0(int Index)
{
  if (mPreviewOccurences > 0)
  {
    mMapPreviewTiles[Index].Index = 0U;
    mMapPreviewTiles[Index].Address = mPreviewAddress;
    mPreviewOccurences--;
    if (mPreviewOccurences == 0U)
    {
      mPreviewAddress += 2;
    }
  }
  else
  {
    if (mPreviewAddress <= mRAMMax)
    {
      tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress);
      mMapPreviewTiles[Index].Index = Data % FormGraphicsExplorer->NbTiles;
      mMapPreviewTiles[Index].Address = mPreviewAddress;

      if (Data == 0U)
      {
        if ((mPreviewAddress+1) <= mRAMMax)
        {
          mPreviewOccurences = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress+1) - 1;
          if (mPreviewOccurences == 0U)
          {
            mPreviewAddress += 2;
          }
        }
      }
      else
      {
        mPreviewAddress++;
      }
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::UpdateMapPreviewDataLinearWordsBigEndian(int Index)
{
  tUShort TileIndex = (tUShort)mReadData(FormMain->NativeCPC, mPreviewAddress);
  TileIndex = TileIndex << 8;
  TileIndex = TileIndex + (tUShort)mReadData(FormMain->NativeCPC, mPreviewAddress+1);

  mMapPreviewTiles[Index].Index = TileIndex % FormGraphicsExplorer->NbTiles;
  mMapPreviewTiles[Index].Address = mPreviewAddress;
  mPreviewAddress += 2;
}
//---------------------------------------------------------------------------

void __fastcall TFormGraphicsGrid::UpdateMapPreviewDataCompressedLinear(int Index)
{
  if (mPreviewOccurences > 0)
  {
    tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress);
    mMapPreviewTiles[Index].Index = Data % FormGraphicsExplorer->NbTiles;
    mMapPreviewTiles[Index].Address = mPreviewAddress;
    mPreviewOccurences--;
    if (mPreviewOccurences == 0U)
    {
      mPreviewAddress++;
    }
  }
  else if (mPreviewIncrements > 0)
  {
    tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress);
    mMapPreviewTiles[Index].Index = Data % FormGraphicsExplorer->NbTiles;
    mMapPreviewTiles[Index].Address = mPreviewAddress++;
    mPreviewIncrements--;
  }
  else
  {
    if (mPreviewAddress <= mRAMMax)
    {
      tUChar Increment = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress++);
      if (mPreviewAddress <= mRAMMax)
      {
        tUChar Data = (tUChar)mReadData(FormMain->NativeCPC, mPreviewAddress);

        mMapPreviewTiles[Index].Index = Data % FormGraphicsExplorer->NbTiles;
        mMapPreviewTiles[Index].Address = mPreviewAddress;

        // Linear ?
        if (Increment & 0x80)
        {
          mPreviewIncrements = (Increment & 0x7f) - 1;
          mPreviewAddress++;
        }
        else
        {
          mPreviewOccurences = Increment - 1;
          if (mPreviewOccurences == 0U)
          {
            mPreviewAddress++;
          }
        }
      }
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::MapperCopy(void)
{
  // Create bitmap
  TBitmap* BitmapP = new TBitmap();

  if (BitmapP)
  {
    int SizeX;
    switch (mMode)
    {
      case 0: SizeX = 4; break;
      case 1: SizeX = 2; break;
      case 2: SizeX = 1; break;
    }

    // Populate bitmap
    if (mMapPreview == false)
    {
      BitmapP->PixelFormat = pf32bit;
      BitmapP->Width = mTileX * 8 * StringGridTiles->ColCount;
      BitmapP->Height = mTileY * 2 * StringGridTiles->RowCount;

      //
      // Copy Current Tiles Display
      //
      for (int TileY = 0; TileY < StringGridTiles->RowCount; TileY++)
      {
        for (int TileX = 0; TileX < StringGridTiles->ColCount; TileX++)
        {
          int TileAddress = mGetTileAddress(FormGraphicsExplorer->TopLeftAddress, TileX, TileY);

          int BaseMapX = TileX * mTileX * 8;
          int BaseMapY = TileY * mTileY * 2;

          for (int ByteY = 0; ByteY < mTileY; ByteY++)
          {
            for (int ByteX = 0; ByteX < mTileX; ByteX++)
            {
              tULong Address = mGetByteAddress(TileAddress, ByteX, ByteY);
              tUChar Data = 0;
              if (Address <= mRAMMax)
              {
                // Read data
                Data = (tUChar)mReadData(FormMain->NativeCPC, Address);
              }

              int PixelX = BaseMapX + (ByteX * 8);
              int PixelY = BaseMapY + (ByteY * 2);
              for (int X=0; X < 8; X++)
              {
                //BitmapP->Canvas->Pixels[X+(ByteX * 8)][Y+(ByteY * 2)] = mGetPixelColor(Data, X/SizeX);
                TColor Color = mGetPixelColor(Data, X/SizeX);
                BitmapP->Canvas->Pixels[PixelX + X][PixelY+0] = Color;
                BitmapP->Canvas->Pixels[PixelX + X][PixelY+1] = Color;
              }
            }
          }
        }
      }

      // Display message
			Application->MessageBox(_T("All shown tiles have been copied to clipboard."), _T("Clipboard"), MB_OK);
    }
    else
    {
      BitmapP->PixelFormat = pf32bit;
      BitmapP->Width = mTileX * 8 * mMapX;
      BitmapP->Height = mTileY * 2 * mMapY;

      Screen->Cursor = crHourGlass;

      //
      // Copy Map
      //
      for (int TileY = 0; TileY < (int)mMapY; TileY++)
      {
        for (int TileX = 0; TileX < (int)mMapX; TileX++)
        {
          tUShort TileIndex = mMapPreviewTiles[TileX + (TileY * mMapX)].Index;
          tULong TileAddress = FormGraphicsExplorer->FirstTileAddress + (TileIndex * mTileX * mTileY);

          int BaseMapX = TileX * mTileX * 8;
          int BaseMapY = TileY * mTileY * 2;

          for (int ByteY = 0; ByteY < mTileY; ByteY++)
          {
            for (int ByteX = 0; ByteX < mTileX; ByteX++)
            {
              tULong Address = mGetByteAddress(TileAddress, ByteX, ByteY);
              tUChar Data = 0;
              if ( (Address <= mRAMMax) && (TileIndex < (tUShort)FormGraphicsExplorer->NbTiles) )
              {
                // Read data
                Data = (tUChar)mReadData(FormMain->NativeCPC, Address);
              }

              int PixelX = BaseMapX + (ByteX * 8);
              int PixelY = BaseMapY + (ByteY * 2);
              for (int X=0; X < 8; X++)
              {
                TColor Color = mGetPixelColor(Data, X/SizeX);
                BitmapP->Canvas->Pixels[PixelX + X][PixelY+0] = Color;
                BitmapP->Canvas->Pixels[PixelX + X][PixelY+1] = Color;
              }
            }
          }
        }
      }

      Screen->Cursor = crDefault;

      // Display message
			Application->MessageBox(_T("Entire map preview has been copied to clipboard."), _T("Clipboard"), MB_OK);
    }

    // Copy Bitmap to clipboard
    WORD AFormat;
    unsigned int AData;
    HPALETTE APalette;
    BitmapP->SaveToClipboardFormat(AFormat, AData, APalette);
    TClipboard *pCB = Clipboard();
    pCB->Clear();
    pCB->SetAsHandle(AFormat, AData);

    delete BitmapP;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::EditTile(void)
{
  // Prepare tile edition
  GraphicsEditorP->ShowHint = ShowHint;
  GraphicsEditorP->Emulator = mEmulatorP;
  GraphicsEditorP->Settings = mSettingsP;
  GraphicsEditorP->TileX = mTileX;
  GraphicsEditorP->TileY = mTileY;
  GraphicsEditorP->Mode = mMode;
  int Pixels=0;
  for (int LoopY=0; LoopY < mTileY; LoopY++)
  {
    for (int LoopX=0; LoopX < mTileX; LoopX++)
    {
      tULong Address = mGetByteAddress(FormGraphicsExplorer->TileAddress, LoopX, LoopY);
      if (Address <= mRAMMax)
      {
        GraphicsEditorP->Data[Pixels++] = (tUChar)mReadData(FormMain->NativeCPC, Address);
      }
      else
      {
        GraphicsEditorP->Data[Pixels++] = 0;
      }
    }
  }

  if (GraphicsEditorP->ShowModal() == mrOk)
  {
    // Apply tile updates
    Pixels=0;
    for (int LoopY=0; LoopY < mTileY; LoopY++)
    {
      for (int LoopX=0; LoopX < mTileX; LoopX++)
      {
        tULong Address = mGetByteAddress(FormGraphicsExplorer->TileAddress, LoopX, LoopY);
        if ((Address <= mRAMMax) && (mWriteData) )
        {
          mWriteData(FormMain->NativeCPC, Address, GraphicsEditorP->Data[Pixels++]);
        }
      }
    }

    RefreshGrid();
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormGraphicsGrid::UpdateTileAddress(void)
{
  if (mMapPreview == true)
  {
    // Update Tile address
    int Index = StringGridTiles->Col + (StringGridTiles->Row * mMapX);
    FormGraphicsExplorer->TileAddress = mMapPreviewTiles[Index].Address;
  }
  else
  {
    // Update tile address
    FormGraphicsExplorer->TileAddress = FormGraphicsExplorer->TopLeftAddress
                  + (StringGridTiles->Col * mTileY * mTileX * StringGridTiles->RowCount)
                  + (StringGridTiles->Row * mTileY * mTileX);
  }

  // Update Scrollbar position
  if (mMapPreview == false)
  {
    ScrollBarAddress->Position = FormGraphicsExplorer->TopLeftAddress;
  }
  else
  {
    ScrollBarAddress->Position = FormGraphicsExplorer->MapStartAddress;
  }
}
//---------------------------------------------------------------------------

