/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2018 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef TFormGraphicsGridH
#define TFormGraphicsGridH
//---------------------------------------------------------------------------
#include <System.Classes.hpp>
#include <Vcl.Controls.hpp>
#include <Vcl.StdCtrls.hpp>
#include <Vcl.Forms.hpp>
#include <Vcl.Grids.hpp>
#include "TFormGraphicsEditor.h"
#include "..\Emulator\TEmulator.h"
#include "..\Settings\TSettings.h"
#include <Vcl.ComCtrls.hpp>



// Events types
typedef tULong __fastcall (__closure *TGraphicsGridGetAddress)(tULong BaseAddress, int X, int Y);
typedef int __fastcall (__closure *TGraphicsGridDrawByte)(tUChar Byte, int X, int Y);
typedef tUChar (*TGraphicsGridReadData)(tNativeCPC* NativeCPC, tULong addr);
typedef void (*TGraphicsGridWriteData)(tNativeCPC* NativeCPC, tULong addr, tUChar data);
typedef void __fastcall (__closure *TGraphicsGridDrawCells)(int ACol, int ARow, TRect &Rect, TGridDrawState State);
typedef void __fastcall (__closure *TGraphicsGridMapPreview)(int Index);
typedef TColor __fastcall (__closure *TGraphicsGridGetPixelColor)(tUChar Data, int X);


typedef struct
{
  tUShort Index;
  tULong  Address;
} TGraphicsGridTilePreview;


//---------------------------------------------------------------------------
class TFormGraphicsGrid : public TForm
{
__published:	// Composants grs par l'EDI
  TStringGrid *StringGridTiles;
  TButton *ButtonByteDecrement;
  TScrollBar *ScrollBarAddress;
  TButton *ButtonByteIncrement;
  void __fastcall FormShow(TObject *Sender);
  void __fastcall FormHide(TObject *Sender);
  void __fastcall FormResize(TObject *Sender);
  void __fastcall StringGridTilesDrawCell(TObject *Sender, int ACol, int ARow, TRect &Rect,
          TGridDrawState State);
  void __fastcall ScrollBarAddressChange(TObject *Sender);
  void __fastcall ButtonByteDecrementClick(TObject *Sender);
  void __fastcall ButtonByteIncrementClick(TObject *Sender);
  void __fastcall StringGridTilesSelectCell(TObject *Sender, int ACol, int ARow,
          bool &CanSelect);
  void __fastcall FormDestroy(TObject *Sender);
  void __fastcall StringGridTilesDblClick(TObject *Sender);
  void __fastcall StringGridTilesMouseLeave(TObject *Sender);
  void __fastcall StringGridTilesMouseMove(TObject *Sender, TShiftState Shift, int X,
          int Y);


private:	// Dclarations utilisateur
  //
  // Members
  //
  TEmulator* mEmulatorP;
  TSettings* mSettingsP;
  bool mMapPreview;
  int mAddressingMode;
  tUChar mMode;
  tUChar mZoom;
  tUChar mTileX;
  tUChar mTileY;
  tULong mMapX;
  tULong mMapY;
  tULong mRAMMax;
  TColor mColor[GA_PALETTE_NBELEMENT];
  TColor mGrayscaleColor[GA_PALETTE_NBELEMENT];
  TColor* mColorsP;
  tULong mPreviewAddress;
  tUChar mPreviewOccurences;
  tUChar mPreviewIncrements;
  TGraphicsGridGetAddress mGetByteAddress;
  TGraphicsGridGetAddress mGetTileAddress;
  TGraphicsGridDrawByte mDrawByte;
  TGraphicsGridReadData mReadData;
  TGraphicsGridWriteData mWriteData;
  TGraphicsGridDrawCells mDrawCells;
  TGraphicsGridMapPreview mDrawMapPreview;
  TGraphicsGridGetPixelColor mGetPixelColor;
  TFormGraphicsEditor* GraphicsEditorP;
  TGraphicsGridTilePreview* mMapPreviewTiles;

  //
  // Methods
  //
  void __fastcall WMMove(TMessage &Message);
  void __fastcall WMSysCommand(TMessage &Message);
  inline void __fastcall SetEmulator(TEmulator* Value) { mEmulatorP=Value; };
  void __fastcall SetSettings(TSettings* Value);
  void __fastcall SetAddressingMode(int Value);
  void __fastcall SetMemory(int Value);
  void __fastcall SetMapperIndexing(int Value);
  void __fastcall SetMapPreview(bool Value);
  void __fastcall SetTileX(tUChar Value);
  void __fastcall SetTileY(tUChar Value);
  void __fastcall SetZoom(tUChar Value);
  void __fastcall SetMapX(tULong Value);
  void __fastcall SetMapY(tULong Value);
  void __fastcall SetMode(int Value);
  void __fastcall SetRAMMax(tULong Value);
  tULong __fastcall GetByteAddressLinear(tULong Address, int X, int Y);
  tULong __fastcall GetTileAddressLinear(tULong Address, int X, int Y);
  tULong __fastcall GetByteAddressScreen(tULong Address, int X, int Y);
  tULong __fastcall GetTileAddressScreen(tULong Address, int X, int Y);
  inline int __fastcall GetGridCol(void) { return StringGridTiles->Col; };
  inline int __fastcall GetGridColCount(void) { return StringGridTiles->ColCount; };
  inline int __fastcall GetGridRow(void) { return StringGridTiles->Row; };
  inline int __fastcall GetGridRowCount(void) { return StringGridTiles->RowCount; };
  void __fastcall ResizeTiles(void);
  void __fastcall UpdatePalette(void);
  void __fastcall DrawStringGridTiles(int ACol, int ARow, TRect &Rect, TGridDrawState State);
  void __fastcall DrawStringGridMapPreview(int ACol, int ARow, TRect &Rect, TGridDrawState State);
  TColor __fastcall GetPixelColorMode0(tUChar Data, int X);
  TColor __fastcall GetPixelColorMode1(tUChar Data, int X);
  TColor __fastcall GetPixelColorMode2(tUChar Data, int X);
  int __fastcall DrawByteMode0(tUChar Data, int X, int Y);
  int __fastcall DrawByteMode1(tUChar Data, int X, int Y);
  int __fastcall DrawByteMode2(tUChar Data, int X, int Y);
  void __fastcall UpdateMapPreview(void);
  void __fastcall UpdateMapPreviewDataLinearBytes(int Index);
  void __fastcall UpdateMapPreviewDataCompressedRL0(int Index);
  void __fastcall UpdateMapPreviewDataLinearWordsBigEndian(int Index);
  void __fastcall UpdateMapPreviewDataCompressedLinear(int Index);

  //
  // Private Properties
  //

public:		// Dclarations utilisateur
  //
  // Public Properties
  //
  __property TEmulator* Emulator = { write=SetEmulator };
  __property TSettings* Settings = { write=SetSettings };
  __property int AddressingMode = { write=SetAddressingMode };
  __property int Memory = { write=SetMemory };
  __property int MapperIndexing = { write=SetMapperIndexing };
  __property bool MapPreview = { write=SetMapPreview };
  __property tUChar TileX = { write=SetTileX };
  __property tUChar TileY = { write=SetTileY };
  __property tUChar Zoom = { write=SetZoom };
  __property tULong MapX = { write=SetMapX };
  __property tULong MapY = { write=SetMapY };
  __property int Mode = { write=SetMode };
  __property tULong RAMMax = { write=SetRAMMax };
  __property int GridCol = { read=GetGridCol };
  __property int GridColCount = { read=GetGridColCount };
  __property int GridRow = { read=GetGridRow };
  __property int GridRowCount = { read=GetGridRowCount };

  //
  // API
  //
  __fastcall TFormGraphicsGrid(TComponent* Owner);
  void __fastcall RefreshGrid(void);
  void __fastcall MapperCopy(void);
  void __fastcall EditTile(void);
  void __fastcall UpdateTileAddress(void);

#pragma option push -vi-
BEGIN_MESSAGE_MAP
  MESSAGE_HANDLER(WM_MOVE, TMessage, WMMove);
  //MESSAGE_HANDLER(WM_SYSCOMMAND, TMessage, WMSysCommand);
END_MESSAGE_MAP(TForm)
#pragma option pop
};
//---------------------------------------------------------------------------
extern PACKAGE TFormGraphicsGrid *FormGraphicsGrid;
//---------------------------------------------------------------------------
#endif
