/*
    CaPriCe Forever - Amstrad CPC 464/664/6128 emulator
    Copyright (C) 2014-2015 by Frdric Coste

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*****************************************************************************
 *
 *   z80dasm.c
 *   Portable Z80 disassembler
 *
 *   Copyright Juergen Buchmueller, all rights reserved.
 *
 *   - This source code is released as freeware for non-commercial purposes.
 *   - You are free to use and redistribute this code in modified or
 *     unmodified form, provided you list me in the credits.
 *   - If you modify this source code, you must add a notice to each modified
 *     source file that it has been changed.  If you're a nice person, you
 *     will clearly mark each change too.  :)
 *   - If you wish to use this for commercial purposes, please contact me at
 *     pullmoll@t-online.de
 *   - The author of this copywritten work reserves the right to change the
 *     terms of its usage and license at any time, including retroactively
 *   - This entire notice must remain in the source code.
 *
 *****************************************************************************/


#include <vcl.h>
#pragma hdrstop
#include "TFormZ80Editor.h"
#include "TFormEditValue.h"
#include "TFormMain.h"
#include "..\..\Common\Z80DASM\z80dasm.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"


//=================================
// Options

//=================================

static const AnsiString cTitle = "Z80 editor";
static const AnsiString cTitlePaused  = " - [PAUSED]";


TFormZ80Editor *FormZ80Editor;

//---------------------------------------------------------------------------
__fastcall TFormZ80Editor::TFormZ80Editor(TComponent* Owner)
  : TForm(Owner)
{
  mSettingsP = NULL;
  mNextInstructions = NULL;
  mPaused = false;

  // Initialise Theme
  Font->Name = "Amstrad CPC464 Regular";
  Font->Size = 6;

  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if ( (Components[Loop]->Name.Pos("PanelFormat") == 1)
         || (Components[Loop]->Name.Pos("PanelReg") == 1)
         || (Components[Loop]->Name.Pos("PanelBreakpoint") == 1) )
    {
      ((TPanel*)Components[Loop])->Font->Color = clBlack;
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::FormShow(TObject *Sender)
{
  if (FormMain == NULL)
  {
    Close();
    return;
  }

  StringGridInstructions->RowCount = StringGridInstructions->Height / StringGridInstructions->DefaultRowHeight;
  StringGridInstructions->ColWidths[0] = 50;
  StringGridInstructions->ColWidths[1] = 100;
  StringGridInstructions->ColWidths[3] = 80;
  StringGridInstructions->ColWidths[2] = StringGridInstructions->Width -
        (StringGridInstructions->ColWidths[0]+StringGridInstructions->ColWidths[1]+StringGridInstructions->ColWidths[3]);

  StringGridStack->RowCount = StringGridStack->Height / StringGridStack->DefaultRowHeight;
  StringGridStack->ColWidths[0] = 50;
  StringGridStack->ColWidths[1] = StringGridStack->Width - StringGridStack->ColWidths[0];

  // Subscribe to step by step event
  FormMain->SubscribeToCPCEvents(OnStepByStepEvent);

  mNextInstructions = new tULong[StringGridInstructions->RowCount];

  UpdateRegisters(FormMain->NativeCPC);
  UpdateFlags(FormMain->NativeCPC);
  UpdateInstructions(FormMain->NativeCPC);
  UpdateStack(FormMain->NativeCPC);
  UpdateInterrupt(FormMain->NativeCPC);
  InitBreakpoint(FormMain->NativeCPC);

  mPaused = FormMain->NativeCPC->paused;
  Caption = Title;

	UpdateRegistersEdition(mPaused);

  if (mSettingsP)
  {
    mSettingsP->Z80FormVisible = true;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormZ80Editor::FormHide(TObject *Sender)
{
  // Unsubscribe to step by step event
  if (FormMain != NULL)
  {
    FormMain->UnsubscribeToCPCEvents(OnStepByStepEvent);
  }

  delete [] mNextInstructions;

  if (mSettingsP)
  {
    mSettingsP->Z80FormVisible = false;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


AnsiString __fastcall TFormZ80Editor::GetTitle(void)
{
  AnsiString Title = cTitle;
  if (mPaused == true) Title += cTitlePaused;
 return Title;
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::SetSettings(TSettings* Value)
{
  mSettingsP = Value;

  if (    (mSettingsP->Z80FormLeft != 0)
       || (mSettingsP->Z80FormTop != 0)
       || (mSettingsP->Z80FormWidth != 0)
       || (mSettingsP->Z80FormHeight != 0) )
  {
    Position = poDesigned;
    Left = mSettingsP->Z80FormLeft;
    Top = mSettingsP->Z80FormTop;
    Width = mSettingsP->Z80FormWidth;
    Height = mSettingsP->Z80FormHeight;
  }

  ShowHint = mSettingsP->ShowHints;

  // Apply theme
  Color = mSettingsP->ThemeBackgroundColor;
  Font->Color = mSettingsP->ThemeText1Color;
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (Components[Loop]->Name.Pos("Label") == 1)
    {
      ((TLabel*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
    else if (Components[Loop]->Name.Pos("PanelIFF") == 1)
    {
      ((TPanel*)Components[Loop])->Font->Color = mSettingsP->ThemeText1Color;
    }
    else if (Components[Loop]->Name.Pos("GroupBox") == 1)
    {
      ((TGroupBox*)Components[Loop])->Font->Color = mSettingsP->ThemeText2Color;
    }
  }

  // Refresh display
  for (int Loop=0; Loop < ComponentCount; Loop++)
  {
    if (((TControl*)Components[Loop])->Visible == true)
    {
      ((TControl*)Components[Loop])->Invalidate();
    }
  }

  if (mSettingsP->HideAll == false)
  {
    Visible = mSettingsP->Z80FormVisible;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::FormResize(TObject *Sender)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->Z80FormWidth = Width;
    mSettingsP->Z80FormHeight = Height;
  }

  StringGridInstructions->RowCount = StringGridInstructions->Height / StringGridInstructions->DefaultRowHeight;
  StringGridInstructions->ColWidths[2] = StringGridInstructions->Width -
        (StringGridInstructions->ColWidths[0]+StringGridInstructions->ColWidths[1]+StringGridInstructions->ColWidths[3]);

  delete [] mNextInstructions;
  mNextInstructions = new tULong[StringGridInstructions->RowCount];

  StringGridStack->RowCount = StringGridStack->Height / StringGridStack->DefaultRowHeight;

  if (FormMain->NativeCPC != NULL)
  {
    UpdateRegisters(FormMain->NativeCPC);
    UpdateFlags(FormMain->NativeCPC);
    UpdateInstructions(FormMain->NativeCPC);
    UpdateStack(FormMain->NativeCPC);
    UpdateInterrupt(FormMain->NativeCPC);
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::WMMove(TMessage &Message)
{
  if ( (Visible == true) && (mSettingsP) )
  {
    mSettingsP->Z80FormLeft = Left;
    mSettingsP->Z80FormTop = Top;
  }

	NOT_USED(Message);
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::PanelFormatRegAFClick(TObject *Sender)
{
  if (PanelFormatRegAF->Caption.IsEmpty() == true)
  {
    PanelFormatRegAF->Caption = "&&";
    tULong CurrentValue = PanelRegA->Caption.ToInt();
    PanelRegA->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegF->Caption.ToInt();
    PanelRegF->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegAF->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegA->Caption);
    PanelRegA->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegF->Caption);
    PanelRegF->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegBCClick(TObject *Sender)
{
  if (PanelFormatRegBC->Caption.IsEmpty() == true)
  {
    PanelFormatRegBC->Caption = "&&";
    tULong CurrentValue = PanelRegB->Caption.ToInt();
    PanelRegB->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegC->Caption.ToInt();
    PanelRegC->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegBC->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegB->Caption);
    PanelRegB->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegC->Caption);
    PanelRegC->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegDEClick(TObject *Sender)
{
  if (PanelFormatRegDE->Caption.IsEmpty() == true)
  {
    PanelFormatRegDE->Caption = "&&";
    tULong CurrentValue = PanelRegD->Caption.ToInt();
    PanelRegD->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegE->Caption.ToInt();
    PanelRegE->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegDE->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegD->Caption);
    PanelRegD->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegE->Caption);
    PanelRegE->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegHLClick(TObject *Sender)
{
  if (PanelFormatRegHL->Caption.IsEmpty() == true)
  {
    PanelFormatRegHL->Caption = "&&";
    tULong CurrentValue = PanelRegH->Caption.ToInt();
    PanelRegH->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegL->Caption.ToInt();
    PanelRegL->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegHL->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegH->Caption);
    PanelRegH->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegL->Caption);
    PanelRegL->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegAFAltClick(
      TObject *Sender)
{
  if (PanelFormatRegAFAlt->Caption.IsEmpty() == true)
  {
    PanelFormatRegAFAlt->Caption = "&&";
    tULong CurrentValue = PanelRegAAlt->Caption.ToInt();
    PanelRegAAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegFAlt->Caption.ToInt();
    PanelRegFAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegAFAlt->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegAAlt->Caption);
    PanelRegAAlt->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegFAlt->Caption);
    PanelRegFAlt->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegBCAltClick(
      TObject *Sender)
{
  if (PanelFormatRegBCAlt->Caption.IsEmpty() == true)
  {
    PanelFormatRegBCAlt->Caption = "&&";
    tULong CurrentValue = PanelRegBAlt->Caption.ToInt();
    PanelRegBAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegCAlt->Caption.ToInt();
    PanelRegCAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegBCAlt->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegBAlt->Caption);
    PanelRegBAlt->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegCAlt->Caption);
    PanelRegCAlt->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegDEAltClick(
      TObject *Sender)
{
  if (PanelFormatRegDEAlt->Caption.IsEmpty() == true)
  {
    PanelFormatRegDEAlt->Caption = "&&";
    tULong CurrentValue = PanelRegDAlt->Caption.ToInt();
    PanelRegDAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegEAlt->Caption.ToInt();
    PanelRegEAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegDEAlt->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegDAlt->Caption);
    PanelRegDAlt->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegEAlt->Caption);
    PanelRegEAlt->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegHLAltClick(
      TObject *Sender)
{
  if (PanelFormatRegHLAlt->Caption.IsEmpty() == true)
  {
    PanelFormatRegHLAlt->Caption = "&&";
    tULong CurrentValue = PanelRegHAlt->Caption.ToInt();
    PanelRegHAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
    CurrentValue = PanelRegLAlt->Caption.ToInt();
    PanelRegLAlt->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegHLAlt->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegHAlt->Caption);
    PanelRegHAlt->Caption = CurrentValue;
    CurrentValue = StrToInt("0x" + PanelRegLAlt->Caption);
    PanelRegLAlt->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegIXClick(TObject *Sender)
{
  if (PanelFormatRegIX->Caption.IsEmpty() == true)
  {
    PanelFormatRegIX->Caption = "&&";
    tULong CurrentValue = PanelRegIX->Caption.ToInt();
    PanelRegIX->Caption = AnsiString::IntToHex(CurrentValue, 4);
  }
  else
  {
    PanelFormatRegIX->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegIX->Caption);
    PanelRegIX->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegIYClick(TObject *Sender)
{
  if (PanelFormatRegIY->Caption.IsEmpty() == true)
  {
    PanelFormatRegIY->Caption = "&&";
    tULong CurrentValue = PanelRegIY->Caption.ToInt();
    PanelRegIY->Caption = AnsiString::IntToHex(CurrentValue, 4);
  }
  else
  {
    PanelFormatRegIY->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegIY->Caption);
    PanelRegIY->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegSPClick(TObject *Sender)
{
  if (PanelFormatRegSP->Caption.IsEmpty() == true)
  {
    PanelFormatRegSP->Caption = "&&";
    tULong CurrentValue = PanelRegSP->Caption.ToInt();
    PanelRegSP->Caption = AnsiString::IntToHex(CurrentValue, 4);
  }
  else
  {
    PanelFormatRegSP->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegSP->Caption);
    PanelRegSP->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegPCClick(TObject *Sender)
{
  if (PanelFormatRegPC->Caption.IsEmpty() == true)
  {
    PanelFormatRegPC->Caption = "&&";
    tULong CurrentValue = PanelRegPC->Caption.ToInt();
    PanelRegPC->Caption = AnsiString::IntToHex(CurrentValue, 4);
  }
  else
  {
    PanelFormatRegPC->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegPC->Caption);
    PanelRegPC->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegIClick(TObject *Sender)
{
  if (PanelFormatRegI->Caption.IsEmpty() == true)
  {
    PanelFormatRegI->Caption = "&&";
    tULong CurrentValue = PanelRegI->Caption.ToInt();
    PanelRegI->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegI->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegI->Caption);
    PanelRegI->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelFormatRegRClick(TObject *Sender)
{
  if (PanelFormatRegR->Caption.IsEmpty() == true)
  {
    PanelFormatRegR->Caption = "&&";
    tULong CurrentValue = PanelRegR->Caption.ToInt();
    PanelRegR->Caption = AnsiString::IntToHex(CurrentValue, 2);
  }
  else
  {
    PanelFormatRegR->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelRegR->Caption);
    PanelRegR->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::ComboBoxFlagChange(TObject *Sender)
{
TComboBox* ComboBox = (TComboBox*)Sender;
tUChar mask = (tUChar)ComboBox->Tag;

  // Update flags
  FormMain->NativeCPC->Z80->Regs.AF.UChar.l &= ~mask;
  if (ComboBox->ItemIndex == 1)
  {
    FormMain->NativeCPC->Z80->Regs.AF.UChar.l |= mask;
  }

  UpdateRegisters(FormMain->NativeCPC);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag0DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag0->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag0->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag0->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag1DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag1->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag1->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag1->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag2DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag2->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag2->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag2->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag3DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag3->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag3->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag3->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag4DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag4->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag4->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag4->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag5DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag5->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag5->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag5->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag6DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag6->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag6->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag6->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
  NOT_USED(Index);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::ComboBoxFlag7DrawItem(TWinControl *Control, int Index,
          TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxFlag7->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxFlag7->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxFlag7->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::PanelRegAClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.AF.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegAF->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegA->Caption = (PanelFormatRegAF->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegFClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.AF.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegAF->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegF->Caption = (PanelFormatRegAF->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
    UpdateFlags(FormMain->NativeCPC);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegBClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.BC.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegBC->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegB->Caption = (PanelFormatRegBC->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegCClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.BC.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegBC->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegC->Caption = (PanelFormatRegBC->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegDClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.DE.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegDE->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegD->Caption = (PanelFormatRegDE->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegEClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.DE.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegDE->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegE->Caption = (PanelFormatRegDE->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegHClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.HL.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegHL->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegH->Caption = (PanelFormatRegHL->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegLClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.HL.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegHL->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegL->Caption = (PanelFormatRegHL->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegAAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.AFx.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegAFAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegAAlt->Caption = (PanelFormatRegAFAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegFAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.AFx.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegAFAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegFAlt->Caption = (PanelFormatRegAFAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegBAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.BCx.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegBCAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegBAlt->Caption = (PanelFormatRegBCAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegCAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.BCx.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegBCAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegCAlt->Caption = (PanelFormatRegBCAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegDAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.DEx.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegDEAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegDAlt->Caption = (PanelFormatRegDEAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegEAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.DEx.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegDEAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegEAlt->Caption = (PanelFormatRegDEAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegHAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.HLx.UChar.h;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegHLAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegHAlt->Caption = (PanelFormatRegHLAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegLAltClick(TObject *Sender)
{
tUChar* Regs = &FormMain->NativeCPC->Z80->Regs.HLx.UChar.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = PanelFormatRegHLAlt->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegLAlt->Caption = (PanelFormatRegHLAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegIXClick(TObject *Sender)
{
tUShort* Regs = &FormMain->NativeCPC->Z80->Regs.IX.UShort.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUShort)*Regs;
  EditValue->Hexadecimal = PanelFormatRegIX->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 4;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 65535;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegIX->Caption = (PanelFormatRegIX->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 4) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegIYClick(TObject *Sender)
{
tUShort* Regs = &FormMain->NativeCPC->Z80->Regs.IY.UShort.l;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUShort)*Regs;
  EditValue->Hexadecimal = PanelFormatRegIY->Caption.IsEmpty() == true ? false : true;
  EditValue->NbDigits = 4;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 65535;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegIY->Caption = (PanelFormatRegIY->Caption.IsEmpty() == false) ? AnsiString::IntToHex(*Regs, 4) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegSPClick(TObject *Sender)
{
tUShort* Regs = &FormMain->NativeCPC->Z80->Regs.SP.UShort.l;
bool ShowHexa = PanelFormatRegSP->Caption.IsEmpty() == true ? false : true;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUShort)*Regs;
  EditValue->Hexadecimal = ShowHexa;
  EditValue->NbDigits = 4;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 65535;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegSP->Caption = (ShowHexa == true) ? AnsiString::IntToHex(*Regs, 4) : AnsiString(*Regs);

    UpdateStack(FormMain->NativeCPC);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegPCClick(TObject *Sender)
{
tUShort* Regs = &FormMain->NativeCPC->Z80->Regs.PC.UShort.l;
bool ShowHexa = PanelFormatRegPC->Caption.IsEmpty() == true ? false : true;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUShort)*Regs;
  EditValue->Hexadecimal = ShowHexa;
  EditValue->NbDigits = 4;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 65535;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegPC->Caption = (ShowHexa == true) ? AnsiString::IntToHex(*Regs, 4) : AnsiString(*Regs);

    UpdateInstructions(FormMain->NativeCPC);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegIClick(TObject *Sender)
{
tULong* Regs = &FormMain->NativeCPC->Z80->Regs.I;
bool ShowHexa = PanelFormatRegI->Caption.IsEmpty() == true ? false : true;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = ShowHexa;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegI->Caption = (ShowHexa == true) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelRegRClick(TObject *Sender)
{
tULong* Regs = &FormMain->NativeCPC->Z80->Regs.R;
bool ShowHexa = PanelFormatRegR->Caption.IsEmpty() == true ? false : true;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (tUChar)*Regs;
  EditValue->Hexadecimal = ShowHexa;
  EditValue->NbDigits = 2;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 255;
  if (EditValue->ShowModal() == mrOk)
  {
    *Regs = EditValue->NewValue;
    PanelRegR->Caption = (ShowHexa == true) ? AnsiString::IntToHex(*Regs, 2) : AnsiString(*Regs);
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::ComboBoxITModeChange(TObject *Sender)
{
  // Update interrupt mode
  FormMain->NativeCPC->Z80->Regs.IM = ComboBoxITMode->ItemIndex;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TFormZ80Editor::ComboBoxITModeDrawItem(TWinControl *Control,
          int Index, TRect &Rect, TOwnerDrawState State)
{
  TCanvas* Canvas = ComboBoxITMode->Canvas;

  if (State.Contains(odSelected))
  {
    Canvas->Brush->Color = clHighlight;
    Canvas->Font->Color = clWhite;
  }
  else
  {
    Canvas->Brush->Color = clWhite;
    Canvas->Font->Color = clBlack;
  }

  // Draw cell
  Canvas->FillRect(Rect);
  AnsiString Text = ComboBoxITMode->Items->Strings[Index];
  int TextHeight = Canvas->TextHeight(Text);
  int TextPos = (ComboBoxITMode->ItemHeight - TextHeight) / 2;
  Canvas->TextOut(Rect.Left+5, Rect.Top+TextPos, Text);
  if (State.Contains(odFocused))
  {
    Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Control);
}
//---------------------------------------------------------------------------

void __fastcall TFormZ80Editor::PanelFormatBreakpoint1Click(
      TObject *Sender)
{
  if (PanelFormatBreakpoint1->Caption.IsEmpty() == true)
  {
    PanelFormatBreakpoint1->Caption = "&&";
    tULong CurrentValue = PanelBreakpoint1->Caption.ToInt();
    PanelBreakpoint1->Caption = AnsiString::IntToHex(CurrentValue, 4);
  }
  else
  {
    PanelFormatBreakpoint1->Caption = "";
    tULong CurrentValue = StrToInt("0x" + PanelBreakpoint1->Caption);
    PanelBreakpoint1->Caption = CurrentValue;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::CheckBoxBreakpoint1Click(
      TObject *Sender)
{
  if (CheckBoxBreakpoint1->Focused())
  {
    tZ80* Z80 = FormMain->NativeCPC->Z80;

    if (CheckBoxBreakpoint1->Checked == true)
    {
      if (PanelFormatBreakpoint1->Caption.IsEmpty() == false)
      {
        Z80->Regs.breakpoint = StrToInt("0x" + PanelBreakpoint1->Caption);
      }
      else
      {
        Z80->Regs.breakpoint = PanelBreakpoint1->Caption.ToInt();
      }
    }
    else
    {
      Z80->Regs.breakpoint = 0xffffffff; // clear break point
    }
    mBreakpoint1 = Z80->Regs.breakpoint;
  }

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::PanelBreakpoint1Click(TObject *Sender)
{
bool ShowHexa = PanelFormatBreakpoint1->Caption.IsEmpty() == true ? false : true;

  TFormEditValue* EditValue = new TFormEditValue(this);
  EditValue->Settings = mSettingsP;
  EditValue->CurrentValue = (ShowHexa == true) ? StrToInt("0x" + PanelBreakpoint1->Caption) :
                                                   PanelBreakpoint1->Caption.ToInt();;
  EditValue->Hexadecimal = ShowHexa;
  EditValue->NbDigits = 4;
  EditValue->MinValue = 0;
  EditValue->MaxValue = 65535;
  if (EditValue->ShowModal() == mrOk)
  {
    tZ80* Z80 = FormMain->NativeCPC->Z80;

    PanelBreakpoint1->Caption = (ShowHexa == true) ? AnsiString::IntToHex(EditValue->NewValue, 4) :
                                                     AnsiString(EditValue->NewValue);

    // Set new breakpoint
    CheckBoxBreakpoint1->Checked = true;
    if (PanelFormatBreakpoint1->Caption.IsEmpty() == false)
    {
      Z80->Regs.breakpoint = StrToInt("0x" + PanelBreakpoint1->Caption);
    }
    else
    {
      Z80->Regs.breakpoint = PanelBreakpoint1->Caption.ToInt();
    }
    mBreakpoint1 = Z80->Regs.breakpoint;
  }
  delete EditValue;

	NOT_USED(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::UpdateRegisters(tNativeCPC* NativeCPC)
{
tZ80* Z80 = NativeCPC->Z80;

  PanelRegA->Caption = (PanelFormatRegAF->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_A, 2) : AnsiString(_A);
  PanelRegF->Caption = (PanelFormatRegAF->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_F, 2) : AnsiString(_F);
  PanelRegB->Caption = (PanelFormatRegBC->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_B, 2) : AnsiString(_B);
  PanelRegC->Caption = (PanelFormatRegBC->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_C, 2) : AnsiString(_C);
  PanelRegD->Caption = (PanelFormatRegDE->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_D, 2) : AnsiString(_D);
  PanelRegE->Caption = (PanelFormatRegDE->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_E, 2) : AnsiString(_E);
  PanelRegH->Caption = (PanelFormatRegHL->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_H, 2) : AnsiString(_H);
  PanelRegL->Caption = (PanelFormatRegHL->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_L, 2) : AnsiString(_L);

  PanelRegAAlt->Caption = (PanelFormatRegAFAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.AFx.UChar.h, 2) : AnsiString(Z80->Regs.AFx.UChar.h);
  PanelRegFAlt->Caption = (PanelFormatRegAFAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.AFx.UChar.l, 2) : AnsiString(Z80->Regs.AFx.UChar.l);
  PanelRegBAlt->Caption = (PanelFormatRegBCAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.BCx.UChar.h, 2) : AnsiString(Z80->Regs.BCx.UChar.h);
  PanelRegCAlt->Caption = (PanelFormatRegBCAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.BCx.UChar.l, 2) : AnsiString(Z80->Regs.BCx.UChar.l);
  PanelRegDAlt->Caption = (PanelFormatRegDEAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.DEx.UChar.h, 2) : AnsiString(Z80->Regs.DEx.UChar.h);
  PanelRegEAlt->Caption = (PanelFormatRegDEAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.DEx.UChar.l, 2) : AnsiString(Z80->Regs.DEx.UChar.l);
  PanelRegHAlt->Caption = (PanelFormatRegHLAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.HLx.UChar.h, 2) : AnsiString(Z80->Regs.HLx.UChar.h);
  PanelRegLAlt->Caption = (PanelFormatRegHLAlt->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.HLx.UChar.l, 2) : AnsiString(Z80->Regs.HLx.UChar.l);

  PanelRegIX->Caption = (PanelFormatRegIX->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_IX, 4) : AnsiString(_IX);
  PanelRegIY->Caption = (PanelFormatRegIY->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_IY, 4) : AnsiString(_IY);
  PanelRegSP->Caption = (PanelFormatRegSP->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_SP, 4) : AnsiString(_SP);
  PanelRegPC->Caption = (PanelFormatRegPC->Caption.IsEmpty() == false) ? AnsiString::IntToHex(_PC, 4) : AnsiString(_PC);

  PanelRegI->Caption = (PanelFormatRegI->Caption.IsEmpty() == false) ? AnsiString::IntToHex((tUChar)_I, 2) : AnsiString((tUChar)_I);
  PanelRegR->Caption = (PanelFormatRegR->Caption.IsEmpty() == false) ? AnsiString::IntToHex((tUChar)_R, 2) : AnsiString((tUChar)_R);
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::UpdateFlags(tNativeCPC* NativeCPC)
{
tUChar flags = NativeCPC->Z80->Regs.AF.UChar.l;

  ComboBoxFlag0->ItemIndex = (flags & 0x01) ? 1 : 0;
  ComboBoxFlag1->ItemIndex = (flags & 0x02) ? 1 : 0;
  ComboBoxFlag2->ItemIndex = (flags & 0x04) ? 1 : 0;
  ComboBoxFlag3->ItemIndex = (flags & 0x08) ? 1 : 0;
  ComboBoxFlag4->ItemIndex = (flags & 0x10) ? 1 : 0;
  ComboBoxFlag5->ItemIndex = (flags & 0x20) ? 1 : 0;
  ComboBoxFlag6->ItemIndex = (flags & 0x40) ? 1 : 0;
  ComboBoxFlag7->ItemIndex = (flags & 0x80) ? 1 : 0;
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::UpdateInstructions(tNativeCPC* NativeCPC)
{
TZ80dasmParam Z80dasmParam;
tULong pc = NativeCPC->Z80->Regs.PC.UShort.l;
tULong addr = pc;
int IndexPC = 0;

  Z80dasmParam.NativeCPC = NativeCPC;
  Z80dasmParam.ReadFuntion = Engine_ReadBankedRead;
  Z80dasmParam.StartAddress = 0;

  // Detect previous instructions
  for (int Loop=0; Loop < StringGridInstructions->RowCount; Loop++)
  {
    if (addr == mNextInstructions[Loop])
    {
      if (Loop > (StringGridInstructions->RowCount/2))
      {
        addr = mNextInstructions[Loop - StringGridInstructions->RowCount/2];
      }
      else
      {
        addr = mNextInstructions[0];
      }
    }
  }

  for (int Loop=0; Loop < StringGridInstructions->RowCount; Loop++)
  {
    bool ShowMemoryContent = false;
    StringGridInstructions->Cells[0][Loop] = AnsiString::IntToHex(addr,4) + ":";

    mNextInstructions[Loop] = addr;

    if (addr == pc)
    {
      IndexPC = Loop;
      ShowMemoryContent = true;
    }

    addr = Z80dasmDecodeInstruction(addr,
                                    &Z80dasmParam);
    StringGridInstructions->Cells[1][Loop] = AnsiString(Z80dasmParam.MachineCode).UpperCase();
    StringGridInstructions->Cells[2][Loop] = AnsiString(Z80dasmParam.Instruction).UpperCase();
    if (ShowMemoryContent)
    {
      StringGridInstructions->Cells[3][Loop] = AnsiString(Z80dasmParam.MemoryContent).UpperCase();
    }
    else
    {
      StringGridInstructions->Cells[3][Loop] = "";
    }
  }

  // Select instruction at PC
  TGridRect Rect;
  Rect.Top = Rect.Bottom = IndexPC;
  Rect.Left = 0;
  Rect.Right = 3;
  StringGridInstructions->Selection = Rect;
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::UpdateStack(tNativeCPC* NativeCPC)
{
tULong addr = NativeCPC->Z80->Regs.SP.UShort.l;

  for (int Loop=0; Loop < StringGridStack->RowCount; Loop++)
  {
    StringGridStack->Cells[0][Loop] = AnsiString::IntToHex(addr,4) + ":";

    tUShort word = Engine_ReadBankedRead(NativeCPC, addr++);
    word += Engine_ReadBankedRead(NativeCPC, addr++) << 8;

    StringGridStack->Cells[1][Loop] = AnsiString::IntToHex(word, 4);
  }

  TGridRect Rect;
  Rect.Top = Rect.Bottom = 0;
  Rect.Left = 0;
  Rect.Right = 1;
  StringGridStack->Selection = Rect;
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::UpdateInterrupt(tNativeCPC* NativeCPC)
{
tZ80* Z80 = NativeCPC->Z80;

  PanelIFF1->Caption = Z80->Regs.IFF1 ? "1" : "0";
  PanelIFF2->Caption = Z80->Regs.IFF2 ? "1" : "0";
  ComboBoxITMode->ItemIndex = Z80->Regs.IM & 0x03;
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::InitBreakpoint(tNativeCPC* NativeCPC)
{
tZ80* Z80 = NativeCPC->Z80;

  //
  // Breakpoint 1
  //
  mBreakpoint1 = Z80->Regs.breakpoint;
  CheckBoxBreakpoint1->Checked = (Z80->Regs.breakpoint != 0xffffffff) ? true : false;
  if (CheckBoxBreakpoint1->Checked == true)
  {
    PanelBreakpoint1->Caption = (PanelFormatBreakpoint1->Caption.IsEmpty() == false) ? AnsiString::IntToHex(Z80->Regs.breakpoint, 4) :
                                                                          AnsiString(Z80->Regs.breakpoint);
  }
  else
  {
    PanelBreakpoint1->Caption = (PanelFormatBreakpoint1->Caption.IsEmpty() == false) ? "FFFF" : "65535";
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormZ80Editor::UpdateBreakpoint(tNativeCPC* NativeCPC)
{
tZ80* Z80 = NativeCPC->Z80;

	//
	// Breakpoint 1
	//
  if (mBreakpoint1 == _PCdword)
  {
    PanelBreakpoint1->Color = clRed;
    PanelBreakpoint1->Font->Color = clWhite;
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormZ80Editor::UpdateRegistersEdition(bool Allow)
{
	ComboBoxITMode->Enabled = Allow;

	PanelRegA->Enabled = Allow;
	PanelRegF->Enabled = Allow;
	PanelRegB->Enabled = Allow;
	PanelRegC->Enabled = Allow;
	PanelRegD->Enabled = Allow;
	PanelRegE->Enabled = Allow;
	PanelRegH->Enabled = Allow;
	PanelRegL->Enabled = Allow;

	PanelRegAAlt->Enabled = Allow;
	PanelRegFAlt->Enabled = Allow;
	PanelRegBAlt->Enabled = Allow;
	PanelRegCAlt->Enabled = Allow;
	PanelRegDAlt->Enabled = Allow;
	PanelRegEAlt->Enabled = Allow;
	PanelRegHAlt->Enabled = Allow;
	PanelRegLAlt->Enabled = Allow;

	PanelRegIX->Enabled = Allow;
	PanelRegIY->Enabled = Allow;
	PanelRegSP->Enabled = Allow;
	PanelRegPC->Enabled = Allow;

	PanelRegI->Enabled = Allow;
	PanelRegR->Enabled = Allow;

	ComboBoxFlag7->Enabled = Allow;
	ComboBoxFlag6->Enabled = Allow;
	ComboBoxFlag5->Enabled = Allow;
	ComboBoxFlag4->Enabled = Allow;
	ComboBoxFlag3->Enabled = Allow;
	ComboBoxFlag2->Enabled = Allow;
	ComboBoxFlag1->Enabled = Allow;
	ComboBoxFlag0->Enabled = Allow;
}
//---------------------------------------------------------------------------


bool __fastcall TFormZ80Editor::OnStepByStepEvent(tULong Condition, tNativeCPC* NativeCPC)
{
tZ80* Z80 = NativeCPC->Z80;
bool Pause = NativeCPC->paused;

  if (Condition & EC_BREAKPOINT)
  {
    UpdateBreakpoint(NativeCPC);
    Pause = true;
  }

  if ( (mPaused == true) && (Pause == false) )
  {
    mPaused = false;
		Caption = Title;

		// Prevent registers edition
		UpdateRegistersEdition(false);

    if (mBreakpoint1 != _PCdword)
    {
      PanelBreakpoint1->Color = clWhite;
      PanelBreakpoint1->Font->Color = clBlack;
    }
	}

  if ( ((Condition & EC_END_INSTRUCTION) == 0) && (Pause == false) )
    return false;

  if (mPaused == false)
  {
    mPaused = true;
    Caption = Title;

		// Allow registers edition
		UpdateRegistersEdition(true);
  }

  if (mBreakpoint1 != _PCdword)
  {
    PanelBreakpoint1->Color = clWhite;
    PanelBreakpoint1->Font->Color = clBlack;
  }

  if (Condition & (EC_END_INSTRUCTION | EC_BREAKPOINT | EC_FDC_EVENT) )
  {
    UpdateRegisters(NativeCPC);
    UpdateFlags(NativeCPC);
    UpdateInstructions(NativeCPC);
    UpdateStack(NativeCPC);
    UpdateInterrupt(NativeCPC);
  }

  return Pause;
}
//---------------------------------------------------------------------------

void __fastcall TFormZ80Editor::StringGridInstructionsDrawCell(TObject *Sender,
          int ACol, int ARow, TRect &Rect, TGridDrawState State)
{
  TStringGrid* StringGrid = (TStringGrid*)Sender;

  if (State.Contains(gdSelected))
  {
    StringGrid->Canvas->Brush->Color = clHighlight;
    StringGrid->Canvas->Font->Color = clWhite;
  }
  else if (ACol == 0)
  {
    StringGrid->Canvas->Brush->Color = Color;
    StringGrid->Canvas->Font->Color = mSettingsP->ThemeText2Color;
  }
  else
  {
    StringGrid->Canvas->Brush->Color = Color;
    StringGrid->Canvas->Font->Color = mSettingsP->ThemeText1Color;
  }

  // Draw cell
  StringGrid->Canvas->FillRect(Rect);
  int TextHeight = StringGrid->Canvas->TextHeight(StringGrid->Cells[ACol][ARow]);
  int TextPos = (StringGrid->RowHeights[ARow] - TextHeight) / 2;
  StringGrid->Canvas->TextOut(Rect.Left+5,
                              Rect.Top+TextPos,
                              StringGrid->Cells[ACol][ARow]);
  if (State.Contains(gdFocused))
  {
    StringGrid->Canvas->DrawFocusRect(Rect);
  }

  NOT_USED(Sender);
}
//---------------------------------------------------------------------------

